"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IronfishCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const errors_1 = require("@oclif/core/lib/errors");
const flags_1 = require("./flags");
const package_1 = require("./package");
const utils_1 = require("./utils");
class IronfishCommand extends core_1.Command {
    constructor(argv, config) {
        super(argv, config);
        /**
         * Set to true when the command is closing so any async things in the command can interrupt and quit
         */
        this.closing = false;
        this.logger = (0, sdk_1.createRootLogger)().withTag(this.ctor.id);
    }
    async run() {
        try {
            await this.start();
        }
        catch (error) {
            if ((0, utils_1.hasUserResponseError)(error)) {
                this.log(error.codeMessage);
                if (error.codeStack) {
                    this.sdk.logger.debug(error.codeStack);
                }
                this.exit(1);
            }
            else if (error instanceof errors_1.ExitError) {
                throw error;
            }
            else if (error instanceof errors_1.CLIError) {
                throw error;
            }
            else if (error instanceof sdk_1.RpcConnectionError) {
                this.log(`Cannot connect to your node, start your node first.`);
            }
            else if (error instanceof sdk_1.DatabaseVersionError) {
                this.log(error.message);
                this.exit(1);
            }
            else if (error instanceof Error) {
                // eslint-disable-next-line no-console
                console.error(sdk_1.ErrorUtils.renderError(error, true));
                this.exit(1);
            }
            else {
                throw error;
            }
        }
        this.exit(0);
    }
    async init() {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-explicit-any
        const commandClass = this.constructor;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        const { flags } = await this.parse(commandClass);
        // Get the flags from the flag object which is unknown
        const dataDirFlag = getFlag(flags, flags_1.DataDirFlagKey);
        const configFlag = getFlag(flags, flags_1.ConfigFlagKey);
        const configOverrides = {};
        const internalOverrides = {};
        const rpcConnectIpcFlag = getFlag(flags, flags_1.RpcUseIpcFlagKey);
        if (typeof rpcConnectIpcFlag === 'boolean' && rpcConnectIpcFlag !== flags_1.RpcUseIpcFlag.default) {
            configOverrides.enableRpcIpc = rpcConnectIpcFlag;
        }
        const rpcConnectTcpFlag = getFlag(flags, flags_1.RpcUseTcpFlagKey);
        if (typeof rpcConnectTcpFlag === 'boolean' && rpcConnectTcpFlag !== flags_1.RpcUseTcpFlag.default) {
            configOverrides.enableRpcTcp = rpcConnectTcpFlag;
        }
        const rpcTcpHostFlag = getFlag(flags, flags_1.RpcTcpHostFlagKey);
        if (typeof rpcTcpHostFlag === 'string') {
            configOverrides.rpcTcpHost = rpcTcpHostFlag;
        }
        const rpcTcpPortFlag = getFlag(flags, flags_1.RpcTcpPortFlagKey);
        if (typeof rpcTcpPortFlag === 'number') {
            configOverrides.rpcTcpPort = rpcTcpPortFlag;
        }
        const rpcConnectHttpFlag = getFlag(flags, flags_1.RpcUseHttpFlagKey);
        if (typeof rpcConnectHttpFlag === 'boolean' &&
            rpcConnectHttpFlag !== flags_1.RpcUseHttpFlag.default) {
            configOverrides.enableRpcHttp = rpcConnectHttpFlag;
        }
        const rpcHttpHostFlag = getFlag(flags, flags_1.RpcHttpHostFlagKey);
        if (typeof rpcHttpHostFlag === 'string') {
            configOverrides.rpcHttpHost = rpcHttpHostFlag;
        }
        const rpcHttpPortFlag = getFlag(flags, flags_1.RpcHttpPortFlagKey);
        if (typeof rpcHttpPortFlag === 'number') {
            configOverrides.rpcHttpPort = rpcHttpPortFlag;
        }
        const rpcTcpTlsFlag = getFlag(flags, flags_1.RpcTcpTlsFlagKey);
        if (typeof rpcTcpTlsFlag === 'boolean' && rpcTcpTlsFlag !== flags_1.RpcTcpTlsFlag.default) {
            configOverrides.enableRpcTls = rpcTcpTlsFlag;
        }
        const verboseFlag = getFlag(flags, flags_1.VerboseFlagKey);
        if (typeof verboseFlag === 'boolean' && verboseFlag !== flags_1.VerboseFlag.default) {
            configOverrides.logLevel = '*:verbose';
        }
        const rpcAuthFlag = getFlag(flags, flags_1.RpcAuthFlagKey);
        if (typeof rpcAuthFlag === 'string') {
            internalOverrides.rpcAuthToken = rpcAuthFlag;
        }
        this.sdk = await sdk_1.IronfishSdk.init({
            pkg: package_1.IronfishCliPKG,
            configOverrides: configOverrides,
            internalOverrides: internalOverrides,
            configName: typeof configFlag === 'string' ? configFlag : undefined,
            dataDir: typeof dataDirFlag === 'string' ? dataDirFlag : undefined,
            logger: this.logger,
        });
    }
    listenForSignals() {
        const signals = ['SIGINT', 'SIGTERM', 'SIGUSR2'];
        for (const signal of signals) {
            const gracefulShutdown = (signal) => {
                if (this.closing) {
                    return;
                }
                // Allow 3 seconds for graceful termination
                setTimeout(() => {
                    this.log('Force closing after 3 seconds');
                    process.exit(1);
                }, 3000).unref();
                this.closing = true;
                const promise = this.closeFromSignal(signal).catch((err) => {
                    this.logger.error(`Failed to close ${sdk_1.ErrorUtils.renderError(err)}`);
                });
                void promise.then(() => {
                    process.exit(0);
                });
            };
            process.once(signal, gracefulShutdown);
        }
    }
    closeFromSignal(signal) {
        throw new Error(`Not implemented closeFromSignal: ${signal}`);
    }
}
exports.IronfishCommand = IronfishCommand;
function getFlag(flags, flag) {
    return typeof flags === 'object' && flags !== null && flag in flags
        ? // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-explicit-any
            flags[flag]
        : null;
}
//# sourceMappingURL=command.js.map