"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const client_cognito_identity_1 = require("@aws-sdk/client-cognito-identity");
const client_s3_1 = require("@aws-sdk/client-s3");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const uuid_1 = require("uuid");
const command_1 = require("../command");
const flags_1 = require("../flags");
const utils_1 = require("../utils");
class Backup extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(Backup);
        const bucket = args.bucket.trim();
        const accessKeyId = flags.accessKeyId;
        const secretAccessKey = flags.secretAccessKey;
        let id = (0, uuid_1.v4)().slice(0, 5);
        const name = this.sdk.config.get('nodeName');
        if (name) {
            id = `${name}.${id}`;
        }
        if (flags.lock) {
            const node = await this.sdk.node({ autoSeed: false });
            await sdk_1.NodeUtils.waitForOpen(node);
        }
        const source = this.sdk.config.dataDir;
        const destDir = this.sdk.config.tempDir;
        await promises_1.default.mkdir(destDir, { recursive: true });
        const destName = `node.${id}.tar.gz`;
        const dest = path_1.default.join(destDir, destName);
        this.log(`Zipping\n    SRC ${source}\n    DST ${dest}\n`);
        core_1.CliUx.ux.action.start(`Zipping ${source}`);
        const excludes = [path_1.default.basename(this.sdk.config.tempDir)];
        if (!flags.chain) {
            excludes.push(path_1.default.basename(path_1.default.dirname(this.sdk.config.chainDatabasePath)));
        }
        if (!flags.wallet) {
            excludes.push(path_1.default.basename(path_1.default.dirname(this.sdk.config.walletDatabasePath)));
        }
        await utils_1.TarUtils.zipDir(source, dest, excludes);
        const stat = await promises_1.default.stat(dest);
        core_1.CliUx.ux.action.stop(`done (${sdk_1.FileUtils.formatFileSize(stat.size)})`);
        core_1.CliUx.ux.action.start(`Uploading to ${bucket}`);
        const s3 = await this.getS3Client(accessKeyId, secretAccessKey);
        await utils_1.S3Utils.uploadToBucket(s3, dest, 'application/x-compressed-tar', bucket, destName, this.logger.withTag('s3'));
        core_1.CliUx.ux.action.stop(`done`);
        core_1.CliUx.ux.action.start(`Removing backup file ${dest}`);
        await promises_1.default.rm(dest);
        core_1.CliUx.ux.action.stop(`done`);
    }
    async getS3Client(accessKeyId, secretAccessKey) {
        const region = 'us-east-1';
        if (accessKeyId && secretAccessKey) {
            return new client_s3_1.S3Client({
                credentials: {
                    accessKeyId,
                    secretAccessKey,
                },
                region,
            });
        }
        const credentials = await this.getCognitoIdentityCredentials();
        return new client_s3_1.S3Client({
            credentials,
            region,
        });
    }
    async getCognitoIdentityCredentials() {
        const identityPoolId = 'us-east-1:3ebc542a-6ac4-4c5d-9558-1621eadd2382';
        const cognito = new client_cognito_identity_1.CognitoIdentity({});
        const identityResponse = await cognito.getId({ IdentityPoolId: identityPoolId });
        const identityId = identityResponse.IdentityId;
        const credentialsResponse = await cognito.getCredentialsForIdentity({
            IdentityId: identityId,
        });
        const cognitoAccessKeyId = credentialsResponse.Credentials?.AccessKeyId;
        const cognitoSecretAccessKey = credentialsResponse.Credentials?.SecretKey;
        const cognitoSessionToken = credentialsResponse.Credentials?.SessionToken;
        sdk_1.Assert.isNotUndefined(cognitoAccessKeyId);
        sdk_1.Assert.isNotUndefined(cognitoSecretAccessKey);
        sdk_1.Assert.isNotUndefined(cognitoSessionToken);
        return {
            accessKeyId: cognitoAccessKeyId,
            secretAccessKey: cognitoSecretAccessKey,
            sessionToken: cognitoSessionToken,
        };
    }
}
Backup.hidden = true;
Backup.description = 'Zip and upload your datadir to an S3 bucket';
Backup.flags = {
    [flags_1.VerboseFlagKey]: flags_1.VerboseFlag,
    [flags_1.DataDirFlagKey]: flags_1.DataDirFlag,
    lock: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Wait for the database to stop being used',
    }),
    chain: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Backup the chain DB',
    }),
    wallet: core_1.Flags.boolean({
        default: false,
        allowNo: true,
        description: 'Backup the wallet',
    }),
    accessKeyId: core_1.Flags.string({
        char: 'a',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'S3 access key ID',
        env: 'AWS_ACCESS_KEY_ID',
        dependsOn: ['secretAccessKey'],
    }),
    secretAccessKey: core_1.Flags.string({
        char: 's',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'S3 secret access key',
        env: 'AWS_SECRET_ACCESS_KEY',
        dependsOn: ['accessKeyId'],
    }),
};
Backup.args = [
    {
        name: 'bucket',
        required: true,
        description: 'The S3 bucket to upload to',
    },
];
exports.default = Backup;
//# sourceMappingURL=backup.js.map