"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const axios_1 = __importDefault(require("axios"));
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const promises_2 = require("stream/promises");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const client_1 = require("../../trusted-setup/client");
class Ceremony extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Ceremony);
        const { host, port } = flags;
        // Pre-make the temp directory to check for access
        const tempDir = this.sdk.config.tempDir;
        await promises_1.default.mkdir(tempDir, { recursive: true });
        const inputPath = path_1.default.join(tempDir, 'params');
        const outputPath = path_1.default.join(tempDir, 'newParams');
        let localHash = null;
        let refreshEtaInterval = null;
        let etaDate = null;
        // Prompt for randomness
        let randomness = await core_1.CliUx.ux.prompt(`If you'd like to contribute your own randomness to the ceremony, type it here, then press Enter. For more information on where this should come from and its importance, please read https://setup.ironfish.network. If you'd like the command to generate some randomness for you, just press Enter`, { required: false });
        randomness = randomness.length ? randomness : null;
        const name = await core_1.CliUx.ux.prompt(`If you'd like to associate a name with this contribution, type it here, then press Enter. Otherwise, to contribute anonymously, just press Enter`, { required: false });
        // Create the client and bind events
        const client = new client_1.CeremonyClient({
            host,
            port,
            logger: this.logger.withTag('ceremonyClient'),
        });
        client.onJoined.on(({ queueLocation, estimate }) => {
            refreshEtaInterval && clearInterval(refreshEtaInterval);
            etaDate = new Date(Date.now() + estimate);
            core_1.CliUx.ux.action.status = renderStatus(queueLocation, etaDate);
            refreshEtaInterval = setInterval(() => {
                core_1.CliUx.ux.action.status = renderStatus(queueLocation, etaDate);
            }, 10 * 1000);
            core_1.CliUx.ux.action.status = `Current position: ${queueLocation}`;
        });
        client.onInitiateContribution.on(async ({ downloadLink, contributionNumber }) => {
            core_1.CliUx.ux.action.stop();
            refreshEtaInterval && clearInterval(refreshEtaInterval);
            this.log(`Starting contribution. You are contributor #${contributionNumber}`);
            core_1.CliUx.ux.action.start(`Downloading the previous contribution to ${inputPath}`);
            const fileHandle = await promises_1.default.open(inputPath, 'w');
            let response;
            try {
                response = await axios_1.default.get(downloadLink, {
                    responseType: 'stream',
                    onDownloadProgress: (p) => {
                        this.log('loaded', p.loaded, 'total', p.total);
                    },
                });
            }
            catch (e) {
                this.error(sdk_1.ErrorUtils.renderError(e));
            }
            await (0, promises_2.pipeline)(response.data, fileHandle.createWriteStream());
            core_1.CliUx.ux.action.stop();
            core_1.CliUx.ux.action.start(`Contributing your randomness`);
            localHash = await (0, rust_nodejs_1.contribute)(inputPath, outputPath, randomness);
            core_1.CliUx.ux.action.stop();
            core_1.CliUx.ux.action.start(`Waiting to upload your contribution`);
            client.contributionComplete();
        });
        client.onInitiateUpload.on(async ({ uploadLink }) => {
            core_1.CliUx.ux.action.stop();
            refreshEtaInterval && clearInterval(refreshEtaInterval);
            core_1.CliUx.ux.action.start(`Uploading your contribution`);
            const fileHandle = await promises_1.default.open(outputPath, 'r');
            const stat = await promises_1.default.stat(outputPath);
            try {
                await axios_1.default.put(uploadLink, fileHandle.createReadStream(), {
                    // Axios requires specifying some max body length when uploading.
                    // Set it high enough that we're not likely to hit it
                    maxBodyLength: 1000000000,
                    headers: {
                        'Content-Type': 'application/octet-stream',
                        'Content-Length': stat.size,
                    },
                });
            }
            catch (e) {
                this.error(sdk_1.ErrorUtils.renderError(e));
            }
            core_1.CliUx.ux.action.stop();
            client.uploadComplete();
            core_1.CliUx.ux.action.start('Contribution uploaded. Waiting for server to verify');
        });
        client.onContributionVerified.on(({ hash, downloadLink, contributionNumber }) => {
            core_1.CliUx.ux.action.stop();
            refreshEtaInterval && clearInterval(refreshEtaInterval);
            if (!localHash) {
                this.log(`Server verified a contribution, but you haven't made a contribution yet. Server-generated hash:`);
                this.log(display256CharacterHash(hash));
                this.error('Please contact the Iron Fish team with this error message.');
            }
            if (hash !== localHash) {
                this.log('Hashes do not match. Locally generated hash:');
                this.log(display256CharacterHash(localHash));
                this.log('Server-generated hash:');
                this.log(display256CharacterHash(hash));
                this.error('Please contact the Iron Fish team with this error message.');
            }
            this.log(`\nThank you for your contribution to the Iron Fish Ceremony. You have successfully contributed at position #${contributionNumber}. The public hash of your contribution is:`);
            this.log(display256CharacterHash(hash));
            this.log(`This hash is a record of your contribution to the Iron Fish parameters, so you should save it to check later. You can view your contributed file at ${downloadLink}.`);
            client.stop(true);
            this.exit(0);
        });
        client.onStopRetry.on(({ error }) => {
            core_1.CliUx.ux.action.stop();
            refreshEtaInterval && clearInterval(refreshEtaInterval);
            this.log(`Stopping contribution: ${error}`);
            client.stop(true);
        });
        // Retry connection until contributions are received
        let connected = false;
        while (!connected) {
            core_1.CliUx.ux.action.start('Connecting');
            const error = await client.start();
            connected = error === null;
            core_1.CliUx.ux.action.stop(error ? `Error connecting: ${error}` : 'done');
            if (!connected) {
                this.log('Unable to connect to contribution server. Retrying in 5 seconds.');
                await sdk_1.PromiseUtils.sleep(5000);
                continue;
            }
            client.join(name, flags.token);
            core_1.CliUx.ux.action.start('Waiting to contribute', undefined, { stdout: true });
            const result = await client.waitForStop();
            connected = result.stopRetries;
            if (!connected) {
                if (core_1.CliUx.ux.action.running) {
                    core_1.CliUx.ux.action.stop('error');
                }
                this.log(`We're sorry, but your contribution either timed out or you lost connection. Attempting to connect again in 5 seconds.`);
                await sdk_1.PromiseUtils.sleep(5000);
            }
        }
    }
}
Ceremony.description = 'Contribute randomness to the Iron Fish trusted setup';
Ceremony.flags = {
    [flags_1.VerboseFlagKey]: flags_1.VerboseFlag,
    [flags_1.DataDirFlagKey]: flags_1.DataDirFlag,
    host: core_1.Flags.string({
        parse: (input) => Promise.resolve(input.trim()),
        default: 'ceremony.ironfish.network',
        description: 'Host address of the ceremony coordination server',
    }),
    port: core_1.Flags.integer({
        default: 9040,
        description: 'Port of the ceremony coordination server',
    }),
    token: core_1.Flags.string({
        required: false,
    }),
};
exports.default = Ceremony;
const renderStatus = (queueLocation, etaDate) => {
    return `Current position: ${queueLocation} ${etaDate
        ? `(Estimated time remaining: ${sdk_1.TimeUtils.renderSpan(etaDate.getTime() - Date.now())})`
        : ''}`;
};
const display256CharacterHash = (hash) => {
    // split string every 8 characters
    let slices = hash.match(/.{1,8}/g) ?? [];
    const output = [];
    for (let i = 0; i < 4; i++) {
        output.push(`\t${slices.slice(0, 4).join(' ')}`);
        slices = slices.slice(4);
    }
    return `\n${output.join('\n')}\n`;
};
//# sourceMappingURL=index.js.map