"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const server_1 = require("../../trusted-setup/server");
const utils_1 = require("../../utils");
const CONTRIBUTE_TIMEOUT_MS = 5 * 60 * 1000;
const UPLOAD_TIMEOUT_MS = 5 * 60 * 1000;
const PRESIGNED_EXPIRATION_SEC = 5 * 60;
const START_DATE = 1680904800000; // Friday, April 07 2023 15:00:00 GMT-0700 (Pacific Daylight Time)
class CeremonyService extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(CeremonyService);
        const DEFAULT_HOST = '0.0.0.0';
        const DEFAULT_PORT = 9040;
        const r2Credentials = await utils_1.S3Utils.getR2Credentials('us-east-1');
        if (r2Credentials === undefined) {
            this.logger.log('Failed getting R2 credentials from AWS');
            this.exit(0);
            return;
        }
        const r2Client = utils_1.S3Utils.getR2S3Client(r2Credentials);
        (0, sdk_1.setLogPrefixFromConfig)(`[%tag%]`);
        const server = new server_1.CeremonyServer({
            logger: (0, sdk_1.createRootLogger)(),
            port: DEFAULT_PORT,
            host: DEFAULT_HOST,
            s3Bucket: flags.bucket,
            downloadPrefix: flags.downloadPrefix,
            s3Client: r2Client,
            tempDir: this.sdk.config.tempDir,
            contributionTimeoutMs: flags.contributionTimeoutMs,
            uploadTimeoutMs: flags.uploadTimeoutMs,
            presignedExpirationSec: flags.presignedExpirationSec,
            startDate: flags.startDate,
            token: flags.token,
            enableIPBanning: !flags.skipIPCheck,
        });
        await server.start();
        await server.waitForStop();
    }
}
CeremonyService.hidden = true;
CeremonyService.description = `
     Start the coordination server for the Iron Fish trusted setup ceremony
   `;
CeremonyService.flags = {
    ...flags_1.RemoteFlags,
    bucket: core_1.Flags.string({
        char: 'b',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'S3/R2 bucket to download and upload params to',
        default: 'ironfish-contributions',
    }),
    downloadPrefix: core_1.Flags.string({
        char: 'b',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'Prefix for contribution download URLs',
        default: 'https://contributions.ironfish.network',
    }),
    contributionTimeoutMs: core_1.Flags.integer({
        required: false,
        description: 'Allowable milliseconds for a contributor to run the contribution script',
        default: CONTRIBUTE_TIMEOUT_MS,
    }),
    uploadTimeoutMs: core_1.Flags.integer({
        required: false,
        description: 'Allowable milliseconds for a contributor to upload their new parameters',
        default: UPLOAD_TIMEOUT_MS,
    }),
    presignedExpirationSec: core_1.Flags.integer({
        required: false,
        description: 'How many seconds the S3/R2 pre-signed upload URL is valid for a contributor',
        default: PRESIGNED_EXPIRATION_SEC,
    }),
    startDate: core_1.Flags.integer({
        required: false,
        description: 'When should the server start accepting contributions',
        default: START_DATE,
    }),
    token: core_1.Flags.string({
        required: true,
    }),
    skipIPCheck: core_1.Flags.boolean({
        required: false,
        description: 'Pass this flag if you want to skip checking for duplicate IPs',
        default: false,
    }),
};
exports.default = CeremonyService;
//# sourceMappingURL=service.js.map