"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const blessed_1 = __importDefault(require("blessed"));
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const package_1 = require("../../package");
class Benchmark extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Benchmark);
        const { blocks } = flags;
        core_1.CliUx.ux.action.start(`Opening node`);
        const node = await this.sdk.node();
        await sdk_1.NodeUtils.waitForOpen(node);
        core_1.CliUx.ux.action.stop('done.');
        let targetDirectory;
        if (!flags.targetdir) {
            await promises_1.default.mkdir(node.config.tempDir, { recursive: true });
            targetDirectory = path_1.default.join(node.config.tempDir);
        }
        else {
            targetDirectory = path_1.default.join(flags.targetdir);
        }
        core_1.CliUx.ux.action.start(`Opening node in ${targetDirectory}`);
        const noLoggingConfig = Object.assign({}, this.sdk.config.overrides);
        noLoggingConfig.logLevel = '*:error';
        const tmpSdk = await sdk_1.IronfishSdk.init({
            pkg: package_1.IronfishCliPKG,
            configOverrides: noLoggingConfig,
            configName: undefined,
            dataDir: targetDirectory,
            logger: this.logger,
        });
        const tempNode = await tmpSdk.node();
        await sdk_1.NodeUtils.waitForOpen(tempNode);
        tempNode.workerPool.start();
        core_1.CliUx.ux.action.stop('done.');
        const startingSequence = tempNode.chain.head.sequence;
        const startingHeader = await node.chain.getHeaderAtSequence(startingSequence);
        const endingSequence = startingSequence + blocks;
        const endingHeader = await node.chain.getHeaderAtSequence(endingSequence);
        if (startingHeader === null) {
            return this.error(`Target chain is longer than source chain`);
        }
        if (endingHeader === null) {
            return this.error(`Chain must have at least ${blocks} blocks`);
        }
        if (!tempNode.chain.head.hash.equals(startingHeader?.hash)) {
            return this.error(`The two chains do not match at sequence ${startingSequence}`);
        }
        let totalMs = 0;
        let totalBlocks = 0;
        let totalTransactions = 0;
        let totalSpends = 0;
        let totalNotes = 0;
        let status = renderStatus(0, 0, 0, 0, 0);
        const screen = blessed_1.default.screen({ smartCSR: true, fullUnicode: true });
        const statusText = blessed_1.default.text();
        screen.append(statusText);
        statusText.setContent(status);
        screen.render();
        const screenInterval = setInterval(() => {
            statusText.setContent(status);
            screen.render();
        }, 1000);
        for await (const currentHeader of node.chain.iterateTo(startingHeader, endingHeader)) {
            const block = await node.chain.getBlock(currentHeader);
            if (block === null) {
                throw new Error('Should have block if we have header');
            }
            const startTime = sdk_1.BenchUtils.start();
            await tempNode.chain.addBlock(block);
            totalMs += sdk_1.BenchUtils.end(startTime);
            totalBlocks += 1;
            totalSpends += block.transactions.reduce((count, tx) => count + tx.spends.length, 0);
            totalNotes += block.transactions.reduce((count, tx) => count + tx.notes.length, 0);
            totalTransactions += block.transactions.length;
            status = renderStatus(totalMs, totalBlocks, totalTransactions, totalSpends, totalNotes, block.header.sequence);
        }
        clearInterval(screenInterval);
        screen.destroy();
        this.log('\n' + status);
        // Check that data is consistent
        if (endingHeader.noteSize === null) {
            return this.error(`Header should have a noteSize`);
        }
        const nodeNotesHash = await node.chain.notes.pastRoot(endingHeader.noteSize);
        const tempNodeNotesHash = await tempNode.chain.notes.rootHash();
        if (!nodeNotesHash.equals(tempNodeNotesHash)) {
            throw new Error('/!\\ Note tree hashes were not consistent /!\\');
        }
        // Clean up the temporary node
        await tempNode.shutdown();
        if (!flags.targetdir) {
            this.log(`\nTemporary directory ${targetDirectory} deleted`);
            await promises_1.default.rm(targetDirectory, { recursive: true });
        }
        else {
            this.log(`\n${blocks} blocks added to ${targetDirectory}`);
        }
    }
}
Benchmark.description = 'Test the performance of the chain by re-importing data from an existing chain';
Benchmark.hidden = true;
Benchmark.flags = {
    ...flags_1.LocalFlags,
    targetdir: core_1.Flags.string({
        char: 't',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'Path to the temporary directory to use to test',
    }),
    blocks: core_1.Flags.integer({
        char: 'b',
        required: false,
        default: 1000,
        description: 'Number of blocks to move from one chain to another',
    }),
};
Benchmark.args = [];
exports.default = Benchmark;
function renderStatus(totalMs, totalBlocks, totalTransactions, totalSpends, totalNotes, sequence) {
    return `\
Current Block        ${sequence ? sequence.toString() : '-'}
Blocks Processed     ${totalBlocks.toString()}
Blocks/sec           ${totalMs ? (totalBlocks / (totalMs / 1000)).toFixed(2) : 0}
Transactions/sec     ${totalMs ? (totalTransactions / (totalMs / 1000)).toFixed(2) : 0}
Spends/sec           ${totalMs ? (totalSpends / (totalMs / 1000)).toFixed(2) : 0}
Notes/sec            ${totalMs ? (totalNotes / (totalMs / 1000)).toFixed(2) : 0}`;
}
//# sourceMappingURL=benchmark.js.map