"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const promises_1 = __importDefault(require("fs/promises"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const snapshot_1 = require("../../snapshot");
class Download extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Download);
        const node = await this.sdk.node();
        await sdk_1.NodeUtils.waitForOpen(node);
        const nodeChainDBVersion = await node.chain.blockchainDb.getVersion();
        const headSequence = node.chain.head.sequence;
        await node.closeDB();
        let downloadedSnapshot;
        if (flags.path) {
            downloadedSnapshot = new snapshot_1.DownloadedSnapshot(this.sdk, flags.path);
        }
        else {
            const networkId = this.sdk.internal.get('networkId');
            const manifestUrl = flags.manifestUrl ?? (0, snapshot_1.getDefaultManifestUrl)(networkId);
            if (!manifestUrl) {
                this.log(`Manifest url for the snapshots are not available for network ID ${networkId}`);
                return this.exit(1);
            }
            let dest = flags.output;
            if (!dest) {
                await promises_1.default.mkdir(this.sdk.config.tempDir, { recursive: true });
                dest = this.sdk.config.tempDir;
            }
            const Downloader = new snapshot_1.SnapshotDownloader(manifestUrl, dest, nodeChainDBVersion);
            const manifest = await Downloader.manifest();
            const fileSize = sdk_1.FileUtils.formatFileSize(manifest.file_size);
            const spaceRequired = sdk_1.FileUtils.formatFileSize(manifest.file_size * 2);
            if (!flags.confirm) {
                const confirm = await core_1.CliUx.ux.confirm(`Download ${fileSize} snapshot to update from block ${headSequence} to ${manifest.block_sequence}? ` +
                    `\nAt least ${spaceRequired} of free disk space is required to download and unzip the snapshot file.` +
                    `\nAre you sure? (Y)es / (N)o`);
                if (!confirm) {
                    this.exit(0);
                }
            }
            const snapshotUrl = await Downloader.snapshotURL();
            const snapshotPath = await Downloader.snapshotPath();
            this.log(`Downloading snapshot from ${snapshotUrl} to ${snapshotPath}`);
            const bar = core_1.CliUx.ux.progress({
                barCompleteChar: '\u2588',
                barIncompleteChar: '\u2591',
                format: 'Downloading snapshot: [{bar}] {percentage}% | {downloadedSize} / {fileSize} | {speed}/s | ETA: {estimate}',
            });
            bar.start(manifest.file_size, 0, {
                fileSize,
                downloadedSize: sdk_1.FileUtils.formatFileSize(0),
                speed: '0',
                estimate: sdk_1.TimeUtils.renderEstimate(0, 0, 0),
            });
            const speed = new sdk_1.Meter();
            speed.start();
            await Downloader.download((prev, curr) => {
                speed.add(curr - prev);
                bar.update(curr, {
                    downloadedSize: sdk_1.FileUtils.formatFileSize(curr),
                    speed: sdk_1.FileUtils.formatFileSize(speed.rate1s),
                    estimate: sdk_1.TimeUtils.renderEstimate(curr, manifest.file_size, speed.rate1m),
                });
            }).catch((error) => {
                bar.stop();
                speed.stop();
                this.logger.error(sdk_1.ErrorUtils.renderError(error));
                this.exit(1);
            });
            const path = await Downloader.verifyChecksum({ cleanup: flags.cleanup });
            if (!path) {
                this.log('Snapshot checksum does not match.');
                return this.exit(0);
            }
            downloadedSnapshot = new snapshot_1.DownloadedSnapshot(this.sdk, path);
        }
        const progressBar = core_1.CliUx.ux.progress({
            barCompleteChar: '\u2588',
            barIncompleteChar: '\u2591',
            format: 'Unzipping snapshot: [{bar}] {percentage}% | {value} / {total} entries | {speed}/s | ETA: {estimate}',
        });
        progressBar.start(0, 0, {
            speed: '0',
            estimate: sdk_1.TimeUtils.renderEstimate(0, 0, 0),
        });
        const speed = new sdk_1.Meter();
        speed.start();
        await downloadedSnapshot.unzip((totalEntries, prevExtracted, currExtracted) => {
            progressBar.setTotal(totalEntries);
            speed.add(currExtracted - prevExtracted);
            progressBar.update(currExtracted, {
                speed: speed.rate1s.toFixed(2),
                estimate: sdk_1.TimeUtils.renderEstimate(currExtracted, totalEntries, speed.rate1m),
            });
        });
        core_1.CliUx.ux.action.start(`Replacing existing chain data at ${downloadedSnapshot.chainDatabasePath} before importing snapshot`);
        await downloadedSnapshot.replaceDatabase();
        core_1.CliUx.ux.action.stop('done');
        if (flags.cleanup) {
            core_1.CliUx.ux.action.start(`Cleaning up snapshot file at ${downloadedSnapshot.file}`);
            await promises_1.default.rm(downloadedSnapshot.file);
            core_1.CliUx.ux.action.stop('done');
        }
    }
}
Download.hidden = false;
Download.description = `Download and import a chain snapshot`;
Download.flags = {
    ...flags_1.LocalFlags,
    manifestUrl: core_1.Flags.string({
        char: 'm',
        parse: (input) => Promise.resolve(input.trim()),
        description: 'Manifest url to download snapshot from',
    }),
    path: core_1.Flags.string({
        char: 'p',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'Path to a downloaded snapshot file to import',
    }),
    output: core_1.Flags.string({
        char: 'o',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'Output folder to download the snapshot file to',
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm download without asking',
    }),
    cleanup: core_1.Flags.boolean({
        default: true,
        description: 'Remove downloaded snapshot file after import',
        allowNo: true,
        hidden: true,
    }),
};
exports.default = Download;
//# sourceMappingURL=download.js.map