"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const promises_1 = __importDefault(require("fs/promises"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
class GenesisAddCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(GenesisAddCommand);
        const node = await this.sdk.node();
        await node.openDB();
        const account = node.wallet.getAccountByName(flags.account);
        if (account === null) {
            this.log(`Account ${flags.account} does not exist, make sure it is imported first`);
            this.exit(0);
            return;
        }
        const totalAmount = sdk_1.CurrencyUtils.decodeIron(flags.totalAmount);
        const csv = await promises_1.default.readFile(flags.allocations, 'utf-8');
        const result = parseAllocationsFile(csv);
        if (!result.ok) {
            this.error(result.error);
        }
        const totalSupply = result.allocations.reduce((prev, cur) => {
            return prev + cur.amountInOre;
        }, 0n);
        if (totalSupply !== totalAmount) {
            this.error(`Allocations file contains ${sdk_1.CurrencyUtils.encodeIron(totalSupply)} $IRON, but --totalAmount expects ${flags.totalAmount} $IRON.`);
        }
        const allocations = result.allocations;
        // Log genesis block info
        this.log(`Genesis block will be modified with the following values in a new transaction:`);
        this.log(`Allocations:`);
        const columns = {
            identity: {
                header: 'ADDRESS',
                get: (row) => row.publicAddress,
            },
            amount: {
                header: 'AMOUNT ($IRON)',
                get: (row) => {
                    return sdk_1.CurrencyUtils.encodeIron(row.amountInOre);
                },
            },
            memo: {
                header: 'MEMO',
                get: (row) => row.memo,
            },
        };
        core_1.CliUx.ux.table(allocations, columns, {
            printLine: (line) => this.log(line),
        });
        // Display duplicates if they exist
        const duplicates = getDuplicates(allocations);
        if (duplicates.length > 0) {
            this.log(`\n/!\\ Allocations contains the following duplicate addresses. This will not cause errors, but may be a mistake. /!\\`);
            for (const duplicate of duplicates) {
                this.log(duplicate);
            }
            this.log('\n');
        }
        // Exit if dry run, otherwise confirm
        if (flags.dry) {
            this.exit(0);
        }
        else {
            const result = await core_1.CliUx.ux.confirm('\nCreate new genesis block? (y)es / (n)o');
            if (!result) {
                this.exit(0);
            }
        }
        this.log('\nBuilding a genesis block...');
        const { block } = await (0, sdk_1.addGenesisTransaction)(node, account, allocations, this.logger);
        this.log(`\nGenesis Block`);
        const serialized = sdk_1.BlockSerde.serialize(block);
        this.log(sdk_1.IJSON.stringify(serialized, '  '));
    }
}
GenesisAddCommand.hidden = true;
GenesisAddCommand.flags = {
    ...flags_1.LocalFlags,
    account: core_1.Flags.string({
        char: 'a',
        required: true,
        description: 'The name of the account to reallocate from',
    }),
    allocations: core_1.Flags.string({
        required: true,
        description: 'A CSV file with the format address,amountInIron,memo containing genesis block allocations',
    }),
    totalAmount: core_1.Flags.string({
        char: 'g',
        required: true,
        description: 'The total prior allocation to the given account',
    }),
    dry: core_1.Flags.boolean({
        default: false,
        description: 'Display genesis block allocations without creating the genesis block',
    }),
};
exports.default = GenesisAddCommand;
const getDuplicates = (allocations) => {
    const duplicateSet = new Set();
    const nonDuplicateSet = new Set();
    for (const alloc of allocations) {
        if (nonDuplicateSet.has(alloc.publicAddress)) {
            duplicateSet.add(alloc.publicAddress);
        }
        else {
            nonDuplicateSet.add(alloc.publicAddress);
        }
    }
    return [...duplicateSet];
};
const parseAllocationsFile = (fileContent) => {
    const allocations = [];
    let lineNum = 0;
    for (const line of fileContent.split(/[\r\n]+/)) {
        lineNum++;
        if (line.trim().length === 0) {
            continue;
        }
        const [address, amountInIron, memo, ...rest] = line.split(',').map((v) => v.trim());
        if (rest.length > 0) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) contains more than 3 values.`,
            };
        }
        // Check address length
        if (!(0, sdk_1.isValidPublicAddress)(address)) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) has an invalid public address.`,
            };
        }
        // Check amount is positive and decodes as $IRON
        const amountInOre = sdk_1.CurrencyUtils.decodeIron(amountInIron);
        if (amountInOre < 0) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) contains a negative $IRON amount.`,
            };
        }
        // Check memo length
        if (Buffer.from(memo).byteLength > rust_nodejs_1.MEMO_LENGTH) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) contains a memo with byte length > ${rust_nodejs_1.MEMO_LENGTH}.`,
            };
        }
        allocations.push({
            publicAddress: address,
            amountInOre: amountInOre,
            memo: memo,
        });
    }
    return { ok: true, allocations };
};
//# sourceMappingURL=genesisadd.js.map