"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const promises_1 = __importDefault(require("fs/promises"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
class GenesisBlockCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(GenesisBlockCommand);
        let target;
        try {
            target = sdk_1.Target.fromDifficulty(BigInt(flags.difficulty));
        }
        catch {
            this.error(`Invalid value for difficulty: ${flags.difficulty}`, { exit: 1 });
        }
        const node = await this.sdk.node({ autoSeed: false });
        await node.openDB();
        if (!node.chain.isEmpty) {
            this.log(`The database ${node.config.chainDatabasePath} must be empty to create a genesis block.`);
            this.exit(0);
        }
        const expectedSupply = sdk_1.CurrencyUtils.decodeIron(flags.genesisSupplyInIron);
        let allocations;
        if (flags.allocations) {
            // If the allocations flag is set, read allocations from a CSV file
            const csv = await promises_1.default.readFile(flags.allocations, 'utf-8');
            const result = parseAllocationsFile(csv);
            if (!result.ok) {
                this.error(result.error);
            }
            const totalSupply = result.allocations.reduce((prev, cur) => {
                return prev + cur.amountInOre;
            }, 0n);
            if (totalSupply !== expectedSupply) {
                this.error(`Allocations file contains ${sdk_1.CurrencyUtils.encodeIron(totalSupply)} $IRON, but --genesisSupplyInIron expects ${flags.genesisSupplyInIron} $IRON.`);
            }
            allocations = result.allocations;
        }
        else {
            // If the allocations flag is not set, create a genesis block with supply belonging to --flags.account
            let account = null;
            if (flags.account !== null) {
                account = node.wallet.getAccountByName(flags.account);
            }
            if (account === null) {
                const name = `IronFishGenesisAccount`; // Faucet depends on the name
                account = await node.wallet.createAccount(name);
                this.log(`Creating account ${account.name} to assign the genesis block to.`);
            }
            allocations = [
                {
                    publicAddress: account.publicAddress,
                    amountInOre: expectedSupply,
                    memo: flags.memo,
                },
            ];
        }
        const info = {
            timestamp: Date.now(),
            target,
            allocations,
        };
        // Log genesis block info
        this.log(`Genesis block will be created with the following values:`);
        this.log(`\nDifficulty: ${target.toDifficulty()}\n`);
        this.log(`Allocations:`);
        const columns = {
            identity: {
                header: 'ADDRESS',
                get: (row) => row.publicAddress,
            },
            amount: {
                header: 'AMOUNT ($IRON)',
                get: (row) => {
                    return sdk_1.CurrencyUtils.encodeIron(row.amountInOre);
                },
            },
            memo: {
                header: 'MEMO',
                get: (row) => row.memo,
            },
        };
        core_1.CliUx.ux.table(info.allocations, columns, {
            printLine: (line) => this.log(line),
        });
        // Display duplicates if they exist
        const duplicates = getDuplicates(allocations);
        if (duplicates.length > 0) {
            this.log(`\n/!\\ Allocations contains the following duplicate addresses. This will not cause errors, but may be a mistake. /!\\`);
            for (const duplicate of duplicates) {
                this.log(duplicate);
            }
            this.log('\n');
        }
        // Exit if dry run, otherwise confirm
        if (flags.dry) {
            this.exit(0);
        }
        else {
            const result = await core_1.CliUx.ux.confirm('\nCreate the genesis block? (y)es / (n)o');
            if (!result) {
                this.exit(0);
            }
        }
        this.log('\nBuilding a genesis block...');
        const { block } = await (0, sdk_1.makeGenesisBlock)(node.chain, info, this.logger);
        this.log(`\nGenesis Block`);
        const serialized = sdk_1.BlockSerde.serialize(block);
        this.log(sdk_1.IJSON.stringify(serialized, '  '));
    }
}
GenesisBlockCommand.description = 'Create and serialize a genesis block';
GenesisBlockCommand.hidden = true;
GenesisBlockCommand.flags = {
    ...flags_1.LocalFlags,
    account: core_1.Flags.string({
        char: 'a',
        required: false,
        default: 'IronFishGenesisAccount',
        description: 'The name of the account to use for keys to assign the genesis block to',
    }),
    difficulty: core_1.Flags.string({
        default: (sdk_1.Target.minDifficulty() * 100n).toString(),
        description: 'The initial difficulty to start the chain with',
    }),
    memo: core_1.Flags.string({
        char: 'm',
        required: false,
        default: 'Genesis Block',
        description: 'The memo of the block',
    }),
    allocations: core_1.Flags.string({
        required: false,
        description: 'A CSV file with the format address,amountInIron,memo containing genesis block allocations',
        exclusive: ['account', 'memo'],
    }),
    genesisSupplyInIron: core_1.Flags.string({
        char: 'g',
        required: true,
        default: '42000000',
        description: 'The amount of coins in the genesis block',
    }),
    dry: core_1.Flags.boolean({
        default: false,
        description: 'Display genesis block allocations without creating the genesis block',
    }),
};
exports.default = GenesisBlockCommand;
const getDuplicates = (allocations) => {
    const duplicateSet = new Set();
    const nonDuplicateSet = new Set();
    for (const alloc of allocations) {
        if (nonDuplicateSet.has(alloc.publicAddress)) {
            duplicateSet.add(alloc.publicAddress);
        }
        else {
            nonDuplicateSet.add(alloc.publicAddress);
        }
    }
    return [...duplicateSet];
};
const parseAllocationsFile = (fileContent) => {
    const allocations = [];
    let lineNum = 0;
    for (const line of fileContent.split(/[\r\n]+/)) {
        lineNum++;
        if (line.trim().length === 0) {
            continue;
        }
        const [address, amountInIron, memo, ...rest] = line.split(',').map((v) => v.trim());
        if (rest.length > 0) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) contains more than 3 values.`,
            };
        }
        // Check address length
        if (!(0, sdk_1.isValidPublicAddress)(address)) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) has an invalid public address.`,
            };
        }
        // Check amount is positive and decodes as $IRON
        const amountInOre = sdk_1.CurrencyUtils.decodeIron(amountInIron);
        if (amountInOre < 0) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) contains a negative $IRON amount.`,
            };
        }
        // Check memo length
        if (Buffer.from(memo).byteLength > rust_nodejs_1.MEMO_LENGTH) {
            return {
                ok: false,
                error: `Line ${lineNum}: (${line}) contains a memo with byte length > ${rust_nodejs_1.MEMO_LENGTH}.`,
            };
        }
        allocations.push({
            publicAddress: address,
            amountInOre: amountInOre,
            memo: memo,
        });
    }
    return { ok: true, allocations };
};
//# sourceMappingURL=genesisblock.js.map