"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rewindChainTo = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
class Rewind extends command_1.IronfishCommand {
    async start() {
        const { args } = await this.parse(Rewind);
        const node = await this.sdk.node();
        await sdk_1.NodeUtils.waitForOpen(node);
        await (0, exports.rewindChainTo)(this, node, Number(args.to), Number(args.from));
    }
}
Rewind.description = 'Rewind the chain database to the given sequence by deleting all blocks with greater sequences';
Rewind.args = [
    {
        name: 'to',
        parse: (input) => Promise.resolve(input.trim()),
        required: true,
        description: 'The block sequence to rewind to',
    },
    {
        name: 'from',
        parse: (input) => Promise.resolve(input.trim()),
        required: false,
        description: 'The sequence to start removing blocks from',
    },
];
Rewind.flags = {
    ...flags_1.LocalFlags,
};
exports.default = Rewind;
const rewindChainTo = async (command, node, to, from) => {
    const chain = node.chain;
    const wallet = node.wallet;
    const sequence = to;
    const fromSequence = from ? Math.max(from, chain.latest.sequence) : chain.latest.sequence;
    const toDisconnect = fromSequence - sequence;
    if (toDisconnect <= 0) {
        command.log(`Chain head currently at ${fromSequence}. Cannot rewind to ${sequence} because it is is greater than the latest sequence in the chain.`);
        command.exit(1);
    }
    command.log(`Chain currently has blocks up to ${fromSequence}. Rewinding ${toDisconnect} blocks to ${sequence}.`);
    await disconnectBlocks(chain, toDisconnect);
    await rewindWalletHead(chain, wallet, sequence);
    await removeBlocks(chain, sequence, fromSequence);
};
exports.rewindChainTo = rewindChainTo;
async function disconnectBlocks(chain, toDisconnect) {
    const bar = getProgressBar('Disconnecting blocks');
    const speed = new sdk_1.Meter();
    bar.start(toDisconnect, 0, {
        speed: '0',
        estimate: sdk_1.TimeUtils.renderEstimate(0, 0, 0),
    });
    speed.start();
    let disconnected = 0;
    while (disconnected < toDisconnect) {
        const headBlock = await chain.getBlock(chain.head);
        sdk_1.Assert.isNotNull(headBlock);
        await chain.blockchainDb.db.transaction(async (tx) => {
            await chain.disconnect(headBlock, tx);
        });
        speed.add(1);
        bar.update(++disconnected, {
            speed: speed.rate1s.toFixed(2),
            estimate: sdk_1.TimeUtils.renderEstimate(disconnected, toDisconnect, speed.rate1m),
        });
    }
    bar.stop();
    speed.stop();
}
async function rewindWalletHead(chain, wallet, sequence) {
    const walletHeadHash = await wallet.getLatestHeadHash();
    if (walletHeadHash) {
        const walletHead = await chain.getHeader(walletHeadHash);
        if (walletHead && walletHead.sequence > sequence) {
            const bar = getProgressBar('Rewiding wallet');
            const speed = new sdk_1.Meter();
            const toRewind = walletHead.sequence - sequence;
            let rewound = 0;
            bar.start(toRewind, 0, {
                speed: '0',
                estimate: sdk_1.TimeUtils.renderEstimate(0, 0, 0),
            });
            speed.start();
            wallet.chainProcessor.onRemove.on((_) => {
                speed.add(1);
                bar.update(++rewound, {
                    speed: speed.rate1s.toFixed(2),
                    estimate: sdk_1.TimeUtils.renderEstimate(rewound, toRewind, speed.rate1m),
                });
            });
            await wallet.updateHead();
            bar.stop();
            speed.stop();
        }
    }
}
async function removeBlocks(chain, sequence, fromSequence) {
    const toRemove = fromSequence - sequence;
    const bar = getProgressBar('Removing blocks');
    const speed = new sdk_1.Meter();
    bar.start(toRemove, 0, {
        speed: '0',
        estimate: sdk_1.TimeUtils.renderEstimate(0, 0, 0),
    });
    speed.start();
    let removed = 0;
    while (fromSequence > sequence) {
        const hashes = await chain.getHashesAtSequence(fromSequence);
        for (const hash of hashes) {
            await chain.removeBlock(hash);
        }
        fromSequence--;
        speed.add(1);
        bar.update(++removed, {
            speed: speed.rate1s.toFixed(2),
            estimate: sdk_1.TimeUtils.renderEstimate(removed, toRemove, speed.rate1m),
        });
    }
    speed.stop();
    bar.stop();
}
function getProgressBar(label) {
    return core_1.CliUx.ux.progress({
        barCompleteChar: '\u2588',
        barIncompleteChar: '\u2591',
        format: `${label}: [{bar}] {percentage}% | {value} / {total} blocks | {speed}/s | ETA: {estimate}`,
    });
}
//# sourceMappingURL=rewind.js.map