"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const child_process_1 = require("child_process");
const os_1 = __importDefault(require("os"));
const v8_1 = require("v8");
const command_1 = require("../command");
const flags_1 = require("../flags");
const SPACE_BUFFER = 8;
class Debug extends command_1.IronfishCommand {
    async start() {
        const node = await this.sdk.node({ autoSeed: false });
        let dbOpen = true;
        try {
            await node.openDB();
        }
        catch (err) {
            if (err instanceof sdk_1.DatabaseIsLockedError) {
                this.log('Database in use, skipping output that requires database.');
                this.log('Stop the node and run the debug command again to show full output.\n');
                dbOpen = false;
            }
            else if (err instanceof sdk_1.DatabaseOpenError) {
                this.log('Database cannot be opened, skipping output that requires database.\n');
                this.log(sdk_1.ErrorUtils.renderError(err, true) + '\n');
                dbOpen = false;
            }
        }
        let output = this.baseOutput(node);
        if (dbOpen) {
            output = new Map([...output, ...(await this.outputRequiringDB(node))]);
        }
        this.display(output);
    }
    baseOutput(node) {
        const cpus = os_1.default.cpus();
        const cpuNames = [...new Set(cpus.map((c) => c.model))];
        const cpuThreads = cpus.length;
        const memTotal = sdk_1.FileUtils.formatMemorySize(os_1.default.totalmem());
        const heapTotal = sdk_1.FileUtils.formatMemorySize((0, v8_1.getHeapStatistics)().total_available_size);
        const telemetryEnabled = this.sdk.config.get('enableTelemetry').toString();
        const assetVerificationEnabled = this.sdk.config.get('enableAssetVerification').toString();
        const nodeName = this.sdk.config.get('nodeName').toString();
        const blockGraffiti = this.sdk.config.get('blockGraffiti').toString();
        let cmdInPath;
        try {
            (0, child_process_1.execSync)('ironfish --help', { stdio: 'ignore' });
            cmdInPath = true;
        }
        catch {
            cmdInPath = false;
        }
        return new Map([
            ['Iron Fish version', `${node.pkg.version} @ ${node.pkg.git}`],
            ['Iron Fish library', `${sdk_1.IronfishPKG.version} @ ${sdk_1.IronfishPKG.git}`],
            ['Operating system', `${os_1.default.type()} ${process.arch}`],
            ['CPU model(s)', `${cpuNames.toString()}`],
            ['CPU threads', `${cpuThreads}`],
            ['RAM total', `${memTotal}`],
            ['Heap total', `${heapTotal}`],
            ['Node version', `${process.version}`],
            ['ironfish in PATH', `${cmdInPath.toString()}`],
            ['Garbage Collector Exposed', `${String(!!global.gc)}`],
            ['Telemetry enabled', `${telemetryEnabled}`],
            ['Asset Verification enabled', `${assetVerificationEnabled}`],
            ['Node name', `${nodeName}`],
            ['Block graffiti', `${blockGraffiti}`],
        ]);
    }
    async outputRequiringDB(node) {
        const output = new Map();
        const headHashes = new Map();
        for await (const { accountId, head } of node.wallet.walletDb.loadHeads()) {
            headHashes.set(accountId, head?.hash ?? null);
        }
        for (const [accountId, headHash] of headHashes.entries()) {
            const account = node.wallet.getAccount(accountId);
            const blockHeader = headHash ? await node.chain.getHeader(headHash) : null;
            const headInChain = !!blockHeader;
            const headSequence = blockHeader?.sequence || 'null';
            const shortId = accountId.slice(0, 6);
            output.set(`Account ${shortId} uuid`, `${accountId}`);
            output.set(`Account ${shortId} name`, `${account?.name || `ACCOUNT NOT FOUND`}`);
            output.set(`Account ${shortId} head hash`, `${headHash ? headHash.toString('hex') : 'NULL'}`);
            output.set(`Account ${shortId} head in chain`, `${headInChain.toString()}`);
            output.set(`Account ${shortId} sequence`, `${headSequence}`);
        }
        return output;
    }
    display(output) {
        // Get the longest key length to determine how big to make the space buffer
        let longestStringLength = 0;
        for (const key of output.keys()) {
            if (key.length > longestStringLength) {
                longestStringLength = key.length;
            }
        }
        const maxKeyWidth = longestStringLength + SPACE_BUFFER;
        output.forEach((value, key) => {
            const spaceWidth = maxKeyWidth - key.length;
            const spaceString = new Array(spaceWidth).join(' ');
            this.log(`${key}${spaceString}${value}`);
        });
    }
}
Debug.description = 'Show debug information to help locate issues';
Debug.hidden = true;
Debug.flags = {
    ...flags_1.LocalFlags,
};
exports.default = Debug;
//# sourceMappingURL=debug.js.map