"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionsCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const { sort: _, ...tableFlags } = core_1.CliUx.ux.table.flags();
const parseMinMax = (input) => {
    if (input.split(':').length === 1) {
        const parsed = parseInt(input);
        return Number.isNaN(parsed) ? undefined : { min: parsed, max: parsed };
    }
    const values = /^([0-9]*)(:([0-9]*))?$/.exec(input);
    if (!values) {
        return undefined;
    }
    const min = ['', undefined].includes(values[1]) ? undefined : parseInt(values[1]);
    const max = ['', undefined].includes(values[3]) ? undefined : parseInt(values[3]);
    return {
        min,
        max,
    };
};
class TransactionsCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(TransactionsCommand);
        let feeRate;
        let fee;
        let expiration;
        let position;
        let expiresIn;
        if (flags.feeRate) {
            feeRate = parseMinMax(flags.feeRate);
            if (feeRate === undefined) {
                this.error('unable to parse flag --feeRate');
            }
        }
        if (flags.fee) {
            fee = parseMinMax(flags.fee);
            if (fee === undefined) {
                this.error('unable to parse flag --fee');
            }
        }
        if (flags.expiration) {
            expiration = parseMinMax(flags.expiration);
            if (expiration === undefined) {
                this.error('unable to parse flag --expiration');
            }
        }
        if (flags.position) {
            position = parseMinMax(flags.position);
            if (position === undefined) {
                this.error('unable to parse flag --position');
            }
        }
        if (flags.expiresIn) {
            expiresIn = parseMinMax(flags.expiresIn);
            if (expiresIn === undefined) {
                this.error('unable to parse flag --expiresIn');
            }
        }
        await this.sdk.client.connect();
        const response = this.sdk.client.mempool.getMempoolTransactionsStream({
            limit: flags.queryLimit,
            feeRate,
            fee,
            expiration,
            position,
            expiresIn,
        });
        const transactions = [];
        for await (const transaction of response.contentStream()) {
            transactions.push(transaction);
        }
        this.log(renderTable(transactions, flags));
        this.exit(0);
    }
}
TransactionsCommand.description = `List transactions in the mempool`;
TransactionsCommand.flags = {
    ...flags_1.RemoteFlags,
    ...tableFlags,
    show: core_1.Flags.integer({
        default: 30,
        description: 'Number of transactions to display on the console',
    }),
    queryLimit: core_1.Flags.integer({
        description: 'Number of transactions to query from the node',
    }),
    feeRate: core_1.Flags.string({
        description: 'Range of values for feeRate given as max:min e.g. `0:5`. A single number indicates equality',
    }),
    fee: core_1.Flags.string({
        description: 'Range of values for fee given as max:min e.g. `0:5`. A single number indicates equality',
    }),
    expiration: core_1.Flags.string({
        aliases: ['exp'],
        description: 'Range of values for expiration sequence given as max:min e.g. `0:5`. A single number indicates equality',
    }),
    position: core_1.Flags.string({
        aliases: ['pos'],
        description: 'Range of values for position in mempool sequence given as max:min e.g. `0:5`. A single number indicates equality',
    }),
    expiresIn: core_1.Flags.string({
        aliases: ['expin'],
        description: 'Range of values for expiration delta from head of chain given as max:min e.g. `0:5`. A single number indicates equality',
    }),
    serializedData: core_1.Flags.boolean({
        aliases: ['full'],
        default: false,
        description: 'Output the entire serialized transaction data. Best used with the --csv flag',
    }),
};
exports.TransactionsCommand = TransactionsCommand;
function renderTable(response, flags) {
    const columns = {
        position: {
            header: 'POSITION',
            minWidth: 4,
            get: (row) => {
                return row.position;
            },
        },
        feeRate: {
            header: 'FEE RATE',
            minWidth: 7,
            get: (row) => {
                return row.feeRate;
            },
        },
        expiration: {
            header: 'EXPIRATION',
            get: (row) => {
                return row.expiration;
            },
        },
        expiresIn: {
            header: 'EXPIRES_IN',
            get: (row) => {
                return row.expiresIn;
            },
        },
        fee: {
            header: 'FEE',
            minWidth: 7,
            get: (row) => {
                return row.fee;
            },
        },
        hash: {
            header: 'HASH',
            minWidth: 65,
            get: (row) => {
                return row.hash;
            },
        },
    };
    if (flags.serializedData) {
        columns['serialized'] = {
            header: 'SERIALIZED',
            minWidth: 2,
            get: (row) => {
                return row.serialized;
            },
        };
    }
    let result = '';
    const limit = flags.csv ? 0 : flags.show;
    core_1.CliUx.ux.table(getRows(response, limit), columns, {
        printLine: (line) => (result += `${String(line)}\n`),
        ...flags,
    });
    if (limit > 0 && response.length > limit) {
        result += ` ... ${response.length - limit} more rows\n`;
    }
    return result;
}
function getRows(response, limit) {
    const transactions = limit > 0 ? response.slice(0, limit) : response;
    return transactions.map(({ serializedTransaction, position, expiresIn }) => {
        const transaction = new sdk_1.Transaction(Buffer.from(serializedTransaction, 'hex'));
        return {
            hash: transaction.hash().toString('hex'),
            feeRate: (0, sdk_1.getFeeRate)(transaction),
            expiration: transaction.expiration(),
            fee: transaction.fee(),
            expiresIn: expiresIn,
            serialized: serializedTransaction,
            position,
        };
    });
}
//# sourceMappingURL=transactions.js.map