"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StartPool = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const dns_1 = __importDefault(require("dns"));
const command_1 = require("../../../command");
const flags_1 = require("../../../flags");
class StartPool extends command_1.IronfishCommand {
    constructor() {
        super(...arguments);
        this.pool = null;
    }
    async start() {
        const { flags } = await this.parse(StartPool);
        const poolName = this.sdk.config.get('poolName');
        const nameByteLen = sdk_1.StringUtils.getByteLength(poolName);
        if (nameByteLen > 18) {
            this.warn(`The provided name ${poolName} has a byte length of ${nameByteLen}`);
            this.warn('It is recommended to keep the pool name below 18 bytes in length to avoid possible work duplication issues');
        }
        const rpc = this.sdk.client;
        this.log(`Starting pool with name ${poolName}`);
        const webhooks = [];
        const discordWebhook = flags.discord ?? this.sdk.config.get('poolDiscordWebhook');
        if (discordWebhook) {
            webhooks.push(new sdk_1.Discord({
                webhook: discordWebhook,
                logger: this.logger,
                explorerBlocksUrl: this.sdk.config.get('explorerBlocksUrl'),
                explorerTransactionsUrl: this.sdk.config.get('explorerTransactionsUrl'),
            }));
            this.log(`Discord enabled: ${discordWebhook}`);
        }
        const larkWebhook = flags.lark ?? this.sdk.config.get('poolLarkWebhook');
        if (larkWebhook) {
            webhooks.push(new sdk_1.Lark({
                webhook: larkWebhook,
                logger: this.logger,
                explorerBlocksUrl: this.sdk.config.get('explorerBlocksUrl'),
                explorerTransactionsUrl: this.sdk.config.get('explorerTransactionsUrl'),
            }));
            this.log(`Lark enabled: ${larkWebhook}`);
        }
        let host = undefined;
        let port = undefined;
        if (flags.host) {
            const parsed = (0, sdk_1.parseUrl)(flags.host);
            if (parsed.hostname) {
                const resolved = await dns_1.default.promises.lookup(parsed.hostname);
                host = resolved.address;
            }
            if (parsed.port) {
                port = parsed.port;
            }
        }
        if (!host) {
            host = this.sdk.config.get('poolHost');
        }
        if (!port) {
            port = this.sdk.config.get('poolPort');
        }
        let tlsOptions = undefined;
        if (flags.tls) {
            const fileSystem = this.sdk.fileSystem;
            const nodeKeyPath = this.sdk.config.get('tlsKeyPath');
            const nodeCertPath = this.sdk.config.get('tlsCertPath');
            tlsOptions = await sdk_1.TlsUtils.getTlsOptions(fileSystem, nodeKeyPath, nodeCertPath, this.logger);
        }
        this.pool = await sdk_1.MiningPool.init({
            config: this.sdk.config,
            logger: this.logger,
            rpc,
            enablePayouts: flags.payouts,
            webhooks: webhooks,
            host: host,
            port: port,
            banning: flags.banning,
            tls: flags.tls,
            tlsOptions: tlsOptions,
        });
        await this.pool.start();
        await this.pool.waitForStop();
    }
    async closeFromSignal() {
        await this.pool?.stop();
    }
}
StartPool.description = `Start a mining pool that connects to a node`;
StartPool.flags = {
    ...flags_1.RemoteFlags,
    discord: core_1.Flags.string({
        description: 'A discord webhook URL to send critical information to',
    }),
    lark: core_1.Flags.string({
        char: 'l',
        description: 'A lark webhook URL to send critical information to',
    }),
    host: core_1.Flags.string({
        char: 'h',
        description: `A host:port listen for stratum connections: ${sdk_1.DEFAULT_POOL_HOST}:${String(sdk_1.DEFAULT_POOL_PORT)}`,
    }),
    payouts: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Whether the pool should payout or not. Useful for solo miners',
    }),
    banning: core_1.Flags.boolean({
        description: 'Whether the pool should ban peers for errors or bad behavior',
        allowNo: true,
    }),
    tls: core_1.Flags.boolean({
        description: 'Whether the pool should listen for connections over tls',
        allowNo: true,
    }),
};
exports.StartPool = StartPool;
//# sourceMappingURL=start.js.map