"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoolStatus = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const blessed_1 = __importDefault(require("blessed"));
const dns_1 = __importDefault(require("dns"));
const command_1 = require("../../../command");
class PoolStatus extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(PoolStatus);
        if (flags.address && !(0, sdk_1.isValidPublicAddress)(flags.address)) {
            this.error('The given public address is not valid, please provide a valid one.');
        }
        let host = this.sdk.config.get('poolHost');
        let port = this.sdk.config.get('poolPort');
        if (flags.pool) {
            const parsed = (0, sdk_1.parseUrl)(flags.pool);
            if (parsed.hostname) {
                const resolved = await dns_1.default.promises.lookup(parsed.hostname);
                host = resolved.address;
            }
            if (parsed.port) {
                port = parsed.port;
            }
        }
        let stratum;
        if (flags.tls) {
            stratum = new sdk_1.StratumTlsClient({ host, port, logger: this.logger });
        }
        else {
            stratum = new sdk_1.StratumTcpClient({ host, port, logger: this.logger });
        }
        if (!flags.follow) {
            stratum.onConnected.on(() => stratum.getStatus(flags.address));
            stratum.onStatus.on((status) => this.log(this.renderStatus(status)));
            stratum.start();
            await (0, sdk_1.waitForEmit)(stratum.onStatus);
            this.exit(0);
        }
        this.logger.pauseLogs();
        const screen = blessed_1.default.screen({ smartCSR: true });
        const statusText = blessed_1.default.text();
        screen.append(statusText);
        stratum.onStatus.on((status) => {
            statusText.setContent(this.renderStatus(status));
            screen.render();
        });
        stratum.start();
        // eslint-disable-next-line no-constant-condition
        while (true) {
            if (!stratum.isConnected()) {
                statusText.setContent(`Not connected to pool ${host}:${port}`);
                screen.render();
                await sdk_1.PromiseUtils.sleep(1000);
                continue;
            }
            stratum.getStatus(flags.address);
            await (0, sdk_1.waitForEmit)(stratum.onStatus);
            await sdk_1.PromiseUtils.sleep(5000);
        }
    }
    renderStatus(status) {
        let result = '';
        result += `Status of mining pool '${status.name}':\n`;
        result += `Miners:                ${status.miners}\n`;
        result += `Hashrate:              ${sdk_1.FileUtils.formatHashRate(status.hashRate)}\n`;
        result += `Shares pending payout: ${status.sharesPending}\n`;
        result += `Clients:               ${status.clients}\n`;
        result += `Bans:                  ${status.bans}\n`;
        if (status.addressStatus) {
            result += `\nMining status for address '${status.addressStatus.publicAddress}':\n`;
            result += `Number of miners:      ${status.addressStatus.miners}\n`;
            result += `Connected miners:      ${status.addressStatus.connectedMiners.join(', ')}\n`;
            result += `Hashrate:              ${sdk_1.FileUtils.formatHashRate(status.addressStatus.hashRate)}\n`;
            result += `Shares pending payout: ${status.addressStatus.sharesPending}`;
        }
        return result;
    }
}
PoolStatus.description = `Show the status of a mining pool`;
PoolStatus.flags = {
    address: core_1.Flags.string({
        char: 'a',
        description: 'The public address for which to retrieve pool share data',
    }),
    pool: core_1.Flags.string({
        char: 'p',
        description: 'The host and port of the mining pool to connect to',
    }),
    follow: core_1.Flags.boolean({
        char: 'f',
        default: false,
        description: 'Follow the status of the mining pool',
    }),
    tls: core_1.Flags.boolean({
        description: 'Connect to pool over tls',
        allowNo: true,
    }),
};
exports.PoolStatus = PoolStatus;
//# sourceMappingURL=status.js.map