"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Miner = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const dns_1 = __importDefault(require("dns"));
const os_1 = __importDefault(require("os"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
class Miner extends command_1.IronfishCommand {
    constructor() {
        super(...arguments);
        this.updateInterval = null;
    }
    async start() {
        const { flags } = await this.parse(Miner);
        if (flags.threads === 0 || flags.threads < -1) {
            throw new Error('--threads must be a positive integer or -1.');
        }
        if (flags.threads === -1) {
            flags.threads = os_1.default.cpus().length;
        }
        const graffiti = this.sdk.config.get('blockGraffiti');
        const batchSize = this.sdk.config.get('minerBatchSize');
        if (flags.pool) {
            let publicAddress = flags.address;
            if (publicAddress == null) {
                const client = await this.sdk.connectRpc();
                const publicKeyResponse = await client.wallet.getAccountPublicKey();
                publicAddress = publicKeyResponse.content.publicKey;
            }
            if (!(0, sdk_1.isValidPublicAddress)(publicAddress)) {
                this.error(`The given public address is not valid, please provide a valid one: ${publicAddress}`);
            }
            let host = this.sdk.config.get('poolHost');
            let port = this.sdk.config.get('poolPort');
            if (flags.pool) {
                const parsed = (0, sdk_1.parseUrl)(flags.pool);
                if (parsed.hostname) {
                    const resolved = await dns_1.default.promises.lookup(parsed.hostname);
                    host = resolved.address;
                }
                if (parsed.port) {
                    port = parsed.port;
                }
            }
            const nameInfo = flags.name ? ` with name ${flags.name}` : '';
            this.log(`Starting to mine with public address: ${publicAddress} at pool ${host}:${port}${nameInfo}`);
            let stratum;
            if (flags.tls) {
                stratum = new sdk_1.StratumTlsClient({ host, port, logger: this.logger });
            }
            else {
                stratum = new sdk_1.StratumTcpClient({ host, port, logger: this.logger });
            }
            const miner = new sdk_1.MiningPoolMiner({
                threadCount: flags.threads,
                publicAddress,
                logger: this.logger,
                batchSize,
                stratum,
                name: flags.name,
            });
            miner.start();
            if (flags.richOutput) {
                this.displayHashrate(miner);
            }
            await miner.waitForStop();
            if (this.updateInterval) {
                clearInterval(this.updateInterval);
            }
        }
        if (!flags.pool) {
            this.log(`Starting to mine with graffiti: ${graffiti}`);
            const rpc = this.sdk.client;
            const miner = new sdk_1.MiningSoloMiner({
                threadCount: flags.threads,
                graffiti: sdk_1.GraffitiUtils.fromString(graffiti),
                logger: this.logger,
                batchSize,
                rpc,
            });
            miner.start();
            if (flags.richOutput) {
                this.displayHashrate(miner);
            }
            await miner.waitForStop();
            if (this.updateInterval) {
                clearInterval(this.updateInterval);
            }
        }
    }
    displayHashrate(miner) {
        core_1.CliUx.ux.action.start(`Hashrate`);
        const updateHashPower = () => {
            const rate = Math.max(0, Math.floor(miner.hashRate.rate5s));
            const formatted = `${sdk_1.FileUtils.formatHashRate(rate)}/s (${rate})`;
            core_1.CliUx.ux.action.status = formatted;
        };
        this.updateInterval = setInterval(updateHashPower, 1000);
    }
}
Miner.description = `Start a miner and subscribe to new blocks for the node`;
Miner.flags = {
    ...flags_1.RemoteFlags,
    threads: core_1.Flags.integer({
        char: 't',
        default: -1,
        description: 'Number of CPU threads to use for mining. -1 will auto-detect based on number of CPU cores.',
    }),
    pool: core_1.Flags.string({
        char: 'p',
        description: 'The host and port of the mining pool to connect to such as 92.191.17.232',
    }),
    name: core_1.Flags.string({
        char: 'n',
        description: 'The miner name distinguishes different miners',
    }),
    address: core_1.Flags.string({
        char: 'a',
        description: 'The public address to receive pool payouts',
    }),
    richOutput: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Enable fancy hashpower display',
    }),
    tls: core_1.Flags.boolean({
        description: 'Connect to pool over tls',
        allowNo: true,
    }),
};
exports.Miner = Miner;
//# sourceMappingURL=start.js.map