"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const blessed_1 = __importDefault(require("blessed"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const STATE_COLUMN_HEADER = 'STATE';
class ListCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(ListCommand);
        flags.sort = flags.sort ?? STATE_COLUMN_HEADER;
        if (!flags.follow) {
            await this.sdk.client.connect();
            const response = await this.sdk.client.peer.getPeers();
            this.log(renderTable(response.content, flags));
            this.exit(0);
        }
        // Console log will create display issues with Blessed
        this.logger.pauseLogs();
        const screen = blessed_1.default.screen({ smartCSR: true, fullUnicode: true });
        const text = blessed_1.default.text();
        screen.append(text);
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const connected = await this.sdk.client.tryConnect();
            if (!connected) {
                text.clearBaseLine(0);
                text.setContent('Connecting...');
                screen.render();
                await sdk_1.PromiseUtils.sleep(1000);
                continue;
            }
            const response = this.sdk.client.peer.getPeersStream();
            for await (const value of response.contentStream()) {
                text.clearBaseLine(0);
                text.setContent(renderTable(value, flags));
                screen.render();
            }
        }
    }
}
ListCommand.description = `List all connected peers`;
ListCommand.flags = {
    ...flags_1.RemoteFlags,
    ...core_1.CliUx.ux.table.flags(),
    follow: core_1.Flags.boolean({
        char: 'f',
        default: false,
        description: 'Follow the peers list live',
    }),
    all: core_1.Flags.boolean({
        default: false,
        description: 'Show all peers, not just connected peers',
    }),
    agents: core_1.Flags.boolean({
        char: 'a',
        default: false,
        description: 'Display peer agents',
    }),
    sequence: core_1.Flags.boolean({
        char: 's',
        default: false,
        description: 'Display peer head sequence',
    }),
    names: core_1.Flags.boolean({
        char: 'n',
        default: false,
        description: 'Display node names',
    }),
    features: core_1.Flags.boolean({
        default: false,
        description: 'Display features that the peers have enabled',
    }),
    host: core_1.Flags.string({
        description: 'Only show peers matching the given host',
    }),
};
exports.ListCommand = ListCommand;
function renderTable(content, flags) {
    let columns = {
        identity: {
            header: 'IDENTITY',
            get: (row) => {
                return row.identity || '-';
            },
        },
    };
    if (flags.names) {
        columns['name'] = {
            header: 'NAME',
            minWidth: 5,
            get: (row) => {
                return row.name || '-';
            },
        };
    }
    if (flags.agents) {
        columns['agents'] = {
            header: 'AGENT',
            minWidth: 5,
            get: (row) => {
                return row.agent || '-';
            },
        };
    }
    if (flags.sequence) {
        columns['sequence'] = {
            header: 'SEQ',
            minWidth: 2,
            get: (row) => {
                return row.sequence || '-';
            },
        };
    }
    if (flags.features) {
        columns['features'] = {
            header: 'FEATURES',
            minWidth: 2,
            get: (row) => {
                if (!row.features) {
                    return '';
                }
                return Object.entries(row.features)
                    .map(([k, v]) => {
                    return v ? k : null;
                })
                    .filter(Boolean)
                    .sort()
                    .join(',');
            },
        };
    }
    columns = {
        ...columns,
        state: {
            header: STATE_COLUMN_HEADER,
            minWidth: 15,
            get: (row) => {
                return row.state + (row.error ? '(!)' : '');
            },
        },
        address: {
            header: 'ADDRESS',
            minWidth: 7,
            get: (row) => {
                let address = '';
                if (row.address) {
                    address += row.address;
                }
                if (row.port) {
                    address += ':' + String(row.port);
                }
                return address;
            },
        },
        connectionDirection: {
            header: 'DIRECTION',
            minWidth: 5,
            extended: true,
            get: (row) => {
                return row.connectionDirection || '-';
            },
        },
        connectionWebSocket: {
            header: 'SOCKET',
            minWidth: 4,
            extended: true,
            get: (row) => {
                return row.connectionWebSocket + (row.connectionWebSocketError ? ' (!)' : '') || '-';
            },
        },
        connectionWebRTC: {
            header: 'RTC',
            minWidth: 5,
            extended: true,
            get: (row) => {
                return row.connectionWebRTC + (row.connectionWebRTCError ? ' (!)' : '') || '-';
            },
        },
        error: {
            header: 'ERROR',
            minWidth: 5,
            extended: true,
            get: (row) => {
                return row.error || '-';
            },
        },
    };
    let peers = content.peers;
    if (!flags.all) {
        peers = peers.filter((p) => p.state === 'CONNECTED');
    }
    if (flags.host) {
        peers = peers.filter((p) => p.address?.includes(flags.host || ''));
    }
    let result = '';
    core_1.CliUx.ux.table(peers, columns, {
        printLine: (line) => (result += `${String(line)}\n`),
        ...flags,
    });
    return result;
}
//# sourceMappingURL=index.js.map