"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const axios_1 = __importDefault(require("axios"));
const fs_1 = __importDefault(require("fs"));
const promises_1 = __importDefault(require("fs/promises"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const command_1 = require("../command");
const flags_1 = require("../flags");
const utils_1 = require("../utils");
const EXTENSION = '.tar.gz';
class Restore extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(Restore);
        const bucket = args.bucket.trim();
        let name = args.name.trim();
        if (!name.endsWith(EXTENSION)) {
            name = name + EXTENSION;
        }
        if (flags.lock) {
            const node = await this.sdk.node({ autoSeed: false });
            await sdk_1.NodeUtils.waitForOpen(node);
            await node.shutdown();
        }
        const workDir = path_1.default.join(os_1.default.tmpdir(), `ironfish.backup`);
        const downloadDir = path_1.default.join(workDir, bucket);
        const downloadTo = path_1.default.join(downloadDir, name);
        const unzipTo = path_1.default.join(downloadDir, path_1.default.basename(downloadTo, EXTENSION));
        const downloadFrom = utils_1.S3Utils.getDownloadUrl(bucket, name, {
            accelerated: false,
            regionCode: 'us-east-1',
        });
        await promises_1.default.rm(workDir, { recursive: true, force: true });
        await promises_1.default.mkdir(downloadDir, { recursive: true });
        await promises_1.default.mkdir(unzipTo, { recursive: true });
        this.log(`Downloading\n    SRC: ${downloadFrom}\n    DST:   ${downloadDir}`);
        const progress = core_1.CliUx.ux.progress({
            format: 'Downloading backup: [{bar}] {percentage}% | ETA: {eta}s',
        });
        progress.start(1);
        await downloadFileTo(downloadFrom, downloadTo, (percent) => {
            progress.update(percent);
        });
        progress.stop();
        this.log(`Unzipping\n    SRC ${downloadTo}\n    DST ${unzipTo}`);
        core_1.CliUx.ux.action.start(`Unzipping ${path_1.default.basename(downloadTo)}`);
        await utils_1.TarUtils.unzipTar(downloadTo, unzipTo);
        core_1.CliUx.ux.action.stop('done\n');
        // We do this because the backup can be created with any datadir name
        // So anything could be inside of the zip file. We want it to match our
        // specified data dir though.
        core_1.CliUx.ux.action.start(`Getting backup name`);
        const backupName = (await promises_1.default.readdir(unzipTo))[0];
        const unzipFrom = path_1.default.join(unzipTo, backupName);
        core_1.CliUx.ux.action.stop(`${backupName}\n`);
        this.log(`Moving\n    SRC ${unzipFrom}\n    DST ${this.sdk.config.dataDir}`);
        core_1.CliUx.ux.action.start(`Moving to ${this.sdk.config.dataDir}`);
        await promises_1.default.rm(this.sdk.config.dataDir, { recursive: true, force: true });
        await promises_1.default.rename(unzipFrom, this.sdk.config.dataDir);
        core_1.CliUx.ux.action.stop(`done\n`);
    }
}
Restore.hidden = true;
Restore.description = 'Download and unzip a datadir from an S3 bucket';
Restore.flags = {
    [flags_1.VerboseFlagKey]: flags_1.VerboseFlag,
    [flags_1.DataDirFlagKey]: flags_1.DataDirFlag,
    lock: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Wait for the database to stop being used',
    }),
};
Restore.args = [
    {
        name: 'bucket',
        required: true,
        description: 'The S3 bucket to upload to',
    },
    {
        name: 'name',
        required: true,
        description: 'The name of the backup from the S3 bucket',
    },
];
exports.default = Restore;
async function downloadFileTo(from, to, onProgress) {
    const [promise, resolve, reject] = sdk_1.PromiseUtils.split();
    const { data, headers } = (await (0, axios_1.default)({
        url: from,
        method: 'GET',
        responseType: 'stream',
    }));
    const writer = fs_1.default.createWriteStream(to);
    const total = headers['content-length'] || 1;
    let current = 0;
    data.on('data', (chunk) => {
        current += chunk.length;
        onProgress(current / total);
    });
    data.on('error', (err) => reject(err));
    data.on('end', () => resolve());
    data.pipe(writer);
    return promise;
}
//# sourceMappingURL=restore.js.map