"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ENABLE_TELEMETRY_CONFIG_KEY = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const node_inspector_1 = __importDefault(require("node:inspector"));
const uuid_1 = require("uuid");
const command_1 = require("../command");
const flags_1 = require("../flags");
const images_1 = require("../images");
exports.ENABLE_TELEMETRY_CONFIG_KEY = 'enableTelemetry';
const DEFAULT_ACCOUNT_NAME = 'default';
class Start extends command_1.IronfishCommand {
    constructor() {
        super(...arguments);
        this.node = null;
        /**
         * This promise is used to wait until start is finished beforer closeFromSignal continues
         * because you can cause errors if you attempt to shutdown while the node is still starting
         * up to reduce shutdown hanging, start should cancel if it detects this.isClosing is true
         * and resolve this promise
         */
        this.startDonePromise = null;
    }
    async start() {
        const [startDonePromise, startDoneResolve] = sdk_1.PromiseUtils.split();
        this.startDonePromise = startDonePromise;
        const { flags } = await this.parse(Start);
        const { bootstrap, forceMining, graffiti, listen, logPeerMessages, name, port, workers, generateNewIdentity, upgrade, networkId, customNetwork, wallet, } = flags;
        if (bootstrap !== undefined) {
            // Parse comma-separated bootstrap nodes
            const bootstrapNodes = bootstrap
                .flatMap((b) => b.split(','))
                .filter(Boolean)
                .map((b) => b.trim());
            this.sdk.config.setOverride('bootstrapNodes', bootstrapNodes);
        }
        if (port !== undefined && port !== this.sdk.config.get('peerPort')) {
            this.sdk.config.setOverride('peerPort', port);
        }
        if (workers !== undefined && workers !== this.sdk.config.get('nodeWorkers')) {
            this.sdk.config.setOverride('nodeWorkers', workers);
        }
        if (graffiti !== undefined && graffiti !== this.sdk.config.get('blockGraffiti')) {
            this.sdk.config.setOverride('blockGraffiti', graffiti);
        }
        if (name !== undefined && name.trim() !== this.sdk.config.get('nodeName')) {
            this.sdk.config.setOverride('nodeName', name.trim());
        }
        if (listen !== undefined && listen !== this.sdk.config.get('enableListenP2P')) {
            this.sdk.config.setOverride('enableListenP2P', listen);
        }
        if (forceMining !== undefined && forceMining !== this.sdk.config.get('miningForce')) {
            this.sdk.config.setOverride('miningForce', forceMining);
        }
        if (wallet !== undefined && wallet !== this.sdk.config.get('enableWallet')) {
            this.sdk.config.setOverride('enableWallet', wallet);
        }
        if (logPeerMessages !== undefined &&
            logPeerMessages !== this.sdk.config.get('logPeerMessages')) {
            this.sdk.config.setOverride('logPeerMessages', logPeerMessages);
        }
        if (generateNewIdentity !== undefined &&
            generateNewIdentity !== this.sdk.config.get('generateNewIdentity')) {
            this.sdk.config.setOverride('generateNewIdentity', generateNewIdentity);
        }
        if (upgrade !== undefined && upgrade !== this.sdk.config.get('databaseMigrate')) {
            this.sdk.config.setOverride('databaseMigrate', upgrade);
        }
        if (networkId !== undefined && customNetwork !== undefined) {
            throw new Error('Cannot specify both the networkId and customNetwork flags at the same time');
        }
        if (networkId !== undefined && networkId !== this.sdk.config.get('networkId')) {
            this.sdk.config.setOverride('networkId', networkId);
        }
        if (customNetwork !== undefined && customNetwork !== this.sdk.config.get('customNetwork')) {
            this.sdk.config.setOverride('customNetwork', customNetwork);
        }
        if (!this.sdk.internal.get('telemetryNodeId')) {
            this.sdk.internal.set('telemetryNodeId', (0, uuid_1.v4)());
            await this.sdk.internal.save();
        }
        const node = await this.sdk.node({ privateIdentity: this.sdk.getPrivateIdentity() });
        const nodeName = this.sdk.config.get('nodeName').trim() || null;
        const blockGraffiti = this.sdk.config.get('blockGraffiti').trim() || null;
        const peerPort = this.sdk.config.get('peerPort');
        const bootstraps = this.sdk.config.getArray('bootstrapNodes');
        const walletEnabled = this.sdk.config.get('enableWallet');
        this.log(`\n${images_1.ONE_FISH_IMAGE}`);
        this.log(`Version       ${node.pkg.version} @ ${node.pkg.git}`);
        this.log(`Node Name     ${nodeName || 'NONE'}`);
        this.log(`Graffiti      ${blockGraffiti || 'NONE'}`);
        this.log(`Peer Identity ${node.peerNetwork.localPeer.publicIdentity}`);
        this.log(`Peer Agent    ${node.peerNetwork.localPeer.agent}`);
        this.log(`Peer Port     ${peerPort}`);
        this.log(`Wallet        ${walletEnabled ? 'ENABLED' : 'DISABLED'}`);
        this.log(`Bootstrap     ${bootstraps.join(',') || 'NONE'}`);
        if (node_inspector_1.default.url()) {
            this.log(`Inspector     ${String(node_inspector_1.default.url())}`);
        }
        this.log(` `);
        await sdk_1.NodeUtils.waitForOpen(node, () => this.closing);
        if (this.closing) {
            return startDoneResolve();
        }
        const headBlock = await node.chain.getBlock(node.chain.head);
        sdk_1.Assert.isNotNull(headBlock);
        const trees = await node.chain.verifier.verifyConnectedBlock(headBlock);
        if (!trees.valid) {
            this.log(`Error starting node: your merkle trees are corrupt: ${String(trees.reason)}.` +
                `\n  1. Run ironfish chain:repair to attempt repair` +
                `\n  2. Delete your database at ${node.config.chainDatabasePath}`);
            this.exit(1);
        }
        const newSecretKey = Buffer.from(node.peerNetwork.localPeer.privateIdentity.secretKey).toString('hex');
        node.internal.set('networkIdentity', newSecretKey);
        await node.internal.save();
        if (node.internal.get('isFirstRun')) {
            await this.firstRun(node);
        }
        await node.start();
        this.node = node;
        startDoneResolve();
        this.listenForSignals();
        await node.waitForShutdown();
    }
    async closeFromSignal(signal) {
        this.log(`Shutting down node after ${signal}`);
        await this.startDonePromise;
        await this.node?.shutdown();
        await this.node?.closeDB();
    }
    /**
     * Information displayed the first time a node is running
     */
    async firstRun(node) {
        this.log('');
        this.log('Thank you for installing the Iron Fish Node.');
        if (!node.config.get(exports.ENABLE_TELEMETRY_CONFIG_KEY)) {
            this.log('');
            this.log('To help improve Iron Fish, opt in to collecting telemetry by running');
            this.log(` > ironfish config:set ${exports.ENABLE_TELEMETRY_CONFIG_KEY} true`);
        }
        if (!node.wallet.getDefaultAccount()) {
            await this.setDefaultAccount(node);
        }
        this.log('');
        node.internal.set('isFirstRun', false);
        node.internal.set('telemetryNodeId', (0, uuid_1.v4)());
        await node.internal.save();
    }
    /**
     * Information displayed if there is no default account for the node
     */
    async setDefaultAccount(node) {
        if (!node.wallet.accountExists(DEFAULT_ACCOUNT_NAME)) {
            const account = await node.wallet.createAccount(DEFAULT_ACCOUNT_NAME, {
                setDefault: true,
            });
            this.log(`New default account created: ${account.name}`);
            this.log(`Account's public address: ${account.publicAddress}`);
        }
        else {
            this.log(`The default account is now: ${DEFAULT_ACCOUNT_NAME}`);
            await node.wallet.setDefaultAccount(DEFAULT_ACCOUNT_NAME);
        }
        this.log('');
        await node.internal.save();
    }
}
Start.description = 'Start the node';
Start.flags = {
    [flags_1.VerboseFlagKey]: flags_1.VerboseFlag,
    [flags_1.ConfigFlagKey]: flags_1.ConfigFlag,
    [flags_1.DataDirFlagKey]: flags_1.DataDirFlag,
    [flags_1.RpcUseIpcFlagKey]: { ...flags_1.RpcUseIpcFlag, allowNo: true },
    [flags_1.RpcUseTcpFlagKey]: { ...flags_1.RpcUseTcpFlag, allowNo: true },
    [flags_1.RpcUseHttpFlagKey]: { ...flags_1.RpcUseHttpFlag, allowNo: true },
    [flags_1.RpcTcpTlsFlagKey]: flags_1.RpcTcpTlsFlag,
    [flags_1.RpcAuthFlagKey]: flags_1.RpcAuthFlag,
    [flags_1.RpcTcpHostFlagKey]: flags_1.RpcTcpHostFlag,
    [flags_1.RpcTcpPortFlagKey]: flags_1.RpcTcpPortFlag,
    [flags_1.RpcHttpHostFlagKey]: flags_1.RpcHttpHostFlag,
    [flags_1.RpcHttpPortFlagKey]: flags_1.RpcHttpPortFlag,
    bootstrap: core_1.Flags.string({
        char: 'b',
        description: 'Comma-separated addresses of bootstrap nodes to connect to',
        multiple: true,
    }),
    port: core_1.Flags.integer({
        char: 'p',
        description: 'Port to run the local ws server on',
    }),
    workers: core_1.Flags.integer({
        description: 'Number of CPU workers to use for long-running operations. 0 disables (likely to cause performance issues), -1 auto-detects based on CPU cores',
    }),
    graffiti: core_1.Flags.string({
        char: 'g',
        default: undefined,
        description: 'Set the graffiti for the node',
    }),
    name: core_1.Flags.string({
        char: 'n',
        description: 'Name for the node',
        hidden: true,
    }),
    listen: core_1.Flags.boolean({
        allowNo: true,
        default: undefined,
        description: 'Disable the web socket listen server',
        hidden: true,
    }),
    forceMining: core_1.Flags.boolean({
        default: undefined,
        description: 'Force mining even if we are not synced',
        hidden: true,
    }),
    logPeerMessages: core_1.Flags.boolean({
        default: undefined,
        description: 'Track all messages sent and received by peers',
        hidden: true,
    }),
    generateNewIdentity: core_1.Flags.boolean({
        default: false,
        description: 'Generate new identity for each new start',
        hidden: true,
    }),
    upgrade: core_1.Flags.boolean({
        allowNo: true,
        description: 'Run migrations when an upgrade is required',
    }),
    networkId: core_1.Flags.integer({
        char: 'i',
        default: undefined,
        description: 'Network ID of an official Iron Fish network to connect to',
    }),
    customNetwork: core_1.Flags.string({
        char: 'c',
        default: undefined,
        description: 'Path to a JSON file containing the network definition of a custom network to connect to',
    }),
    wallet: core_1.Flags.boolean({
        allowNo: true,
        default: true,
        description: `Enable the node's wallet to scan transactions and decrypt notes from the blockchain`,
    }),
};
exports.default = Start;
//# sourceMappingURL=start.js.map