"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const blessed_1 = __importDefault(require("blessed"));
const command_1 = require("../command");
const flags_1 = require("../flags");
class Status extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Status);
        if (!flags.follow) {
            const client = await this.sdk.connectRpc();
            const response = await client.node.getStatus();
            this.log(renderStatus(response.content, flags.all));
            this.exit(0);
        }
        // Console log will create display issues with Blessed
        this.logger.pauseLogs();
        const screen = blessed_1.default.screen({ smartCSR: true, fullUnicode: true });
        const statusText = blessed_1.default.text();
        screen.append(statusText);
        let previousResponse = null;
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const connected = await this.sdk.client.tryConnect();
            if (!connected) {
                statusText.clearBaseLine(0);
                if (previousResponse) {
                    statusText.setContent(renderStatus(previousResponse, flags.all));
                    statusText.insertTop('Node: Disconnected \n');
                }
                else {
                    statusText.setContent('Node: STOPPED');
                }
                screen.render();
                await sdk_1.PromiseUtils.sleep(1000);
                continue;
            }
            // eslint-disable-next-line no-constant-condition
            while (true) {
                try {
                    previousResponse = (await this.sdk.client.node.getStatus()).content;
                }
                catch (e) {
                    break;
                }
                statusText.clearBaseLine(0);
                statusText.setContent(renderStatus(previousResponse, flags.all));
                screen.render();
                await sdk_1.PromiseUtils.sleep(1000);
            }
        }
    }
}
Status.description = 'Show the status of the node';
Status.flags = {
    ...flags_1.RemoteFlags,
    follow: core_1.Flags.boolean({
        char: 'f',
        default: false,
        description: 'Follow the status of the node live',
    }),
    all: core_1.Flags.boolean({
        default: false,
        description: 'show all status information',
    }),
};
exports.default = Status;
function renderStatus(content, debugOutput) {
    const nodeStatus = `${content.node.status.toUpperCase()}`;
    let blockSyncerStatus = content.blockSyncer.status.toString().toUpperCase();
    const blockSyncerStatusDetails = [];
    let telemetryStatus = `${content.telemetry.status.toUpperCase()}`;
    sdk_1.Assert.isNotUndefined(content.blockSyncer.syncing);
    const avgTimeToAddBlock = content.blockSyncer.syncing.blockSpeed;
    if (content.blockSyncer.status === 'syncing') {
        blockSyncerStatusDetails.push(`${content.blockSyncer.syncing.speed} blocks synced/sec, ${content.blockSyncer.syncing.downloadSpeed} blocks downloaded/sec`);
    }
    if (avgTimeToAddBlock) {
        blockSyncerStatusDetails.push(`${(1000 / avgTimeToAddBlock).toFixed(2)} blocks added/sec`);
    }
    if (!content.blockchain.synced) {
        blockSyncerStatusDetails.push(`progress: ${(content.blockSyncer.syncing.progress * 100).toFixed(2)}%`);
    }
    blockSyncerStatus += blockSyncerStatusDetails.length
        ? ` - ${blockSyncerStatusDetails.join(', ')}`
        : '';
    if (content.telemetry.status === 'started') {
        telemetryStatus += ` - ${content.telemetry.submitted} <- ${content.telemetry.pending} pending`;
    }
    const blockGraffiti = `${content.miningDirector.blockGraffiti}`;
    const network = (0, sdk_1.defaultNetworkName)(content.node.networkId) || content.node.networkId.toString();
    const peerNetworkStatus = `${content.peerNetwork.isReady ? 'CONNECTED' : 'WAITING'} - In: ${sdk_1.FileUtils.formatFileSize(content.peerNetwork.inboundTraffic)}/s, Out: ${sdk_1.FileUtils.formatFileSize(content.peerNetwork.outboundTraffic)}/s, peers ${content.peerNetwork.peers}`;
    const blockchainStatus = `${content.blockchain.head.hash} (${content.blockchain.head.sequence}), Since HEAD: ${sdk_1.TimeUtils.renderSpan(Date.now() - content.blockchain.headTimestamp)} (${content.blockchain.synced ? 'SYNCED' : 'NOT SYNCED'})`;
    let miningDirectorStatus = `${content.miningDirector.status.toUpperCase()} - ${content.miningDirector.miners} miners, ${content.miningDirector.blocks} mined`;
    if (debugOutput) {
        miningDirectorStatus += `, get txs: ${sdk_1.TimeUtils.renderSpan(content.miningDirector.newBlockTransactionsSpeed)}, block: ${sdk_1.TimeUtils.renderSpan(content.blockchain.newBlockSpeed)}, empty template: ${sdk_1.TimeUtils.renderSpan(content.miningDirector.newEmptyBlockTemplateSpeed)}, full template: ${sdk_1.TimeUtils.renderSpan(content.miningDirector.newBlockTemplateSpeed)}`;
    }
    const memPoolStorage = sdk_1.FileUtils.formatMemorySize(content.memPool.sizeBytes);
    const memPoolMaxStorage = sdk_1.FileUtils.formatMemorySize(content.memPool.maxSizeBytes);
    const memPoolSaturationPercentage = ((content.memPool.sizeBytes / content.memPool.maxSizeBytes) *
        100).toFixed(2);
    const memPoolStatus = `Count: ${content.memPool.size} tx, Bytes: ${memPoolStorage} / ${memPoolMaxStorage} (${memPoolSaturationPercentage}%), Evictions: ${content.memPool.evictions}`;
    let workersStatus = `${content.workers.started ? 'STARTED' : 'STOPPED'}`;
    if (content.workers.started) {
        workersStatus += ` - ${content.workers.queued} -> ${content.workers.executing} / ${content.workers.capacity} - ${content.workers.change} jobs Δ, ${content.workers.speed} jobs/s`;
    }
    const heapTotal = sdk_1.FileUtils.formatMemorySize(content.memory.heapTotal);
    const heapUsed = sdk_1.FileUtils.formatMemorySize(content.memory.heapUsed);
    const heapMax = sdk_1.FileUtils.formatMemorySize(content.memory.heapMax);
    const rss = sdk_1.FileUtils.formatMemorySize(content.memory.rss);
    const memFree = sdk_1.FileUtils.formatMemorySize(content.memory.memFree);
    const memoryStatus = `Heap: ${heapUsed} -> ${heapTotal} / ${heapMax} (${((content.memory.heapUsed / content.memory.heapMax) *
        100).toFixed(1)}%), RSS: ${rss} (${((content.memory.rss / content.memory.memTotal) *
        100).toFixed(1)}%), Free: ${memFree} (${((1 - content.memory.memFree / content.memory.memTotal) *
        100).toFixed(1)}%)`;
    let accountStatus;
    if (content.accounts.scanning === undefined) {
        accountStatus = `${content.accounts.head.hash}`;
        if (content.accounts.head.sequence !== -1) {
            accountStatus += ` (${content.accounts.head.sequence})`;
        }
    }
    else {
        accountStatus = `SCANNING`;
        if (content.accounts.scanning.sequence !== -1) {
            accountStatus += ` - ${content.accounts.scanning.sequence} / ${content.accounts.scanning.endSequence}`;
        }
    }
    if (!content.accounts.enabled) {
        accountStatus += accountStatus.length === 0 ? `DISABLED` : ` (DISABLED)`;
    }
    const cores = `Cores: ${content.cpu.cores}`;
    const current = `Current: ${content.cpu.percentCurrent.toFixed(1)}%`;
    const rollingAvg = `Rolling Avg: ${content.cpu.percentRollingAvg.toFixed(1)}%`;
    const cpuStatus = debugOutput
        ? [cores, current, rollingAvg].join(', ')
        : [cores, current].join(', ');
    return `\
Version              ${content.node.version} @ ${content.node.git}
Node                 ${nodeStatus}
Node Name            ${content.node.nodeName}
Peer ID              ${content.peerNetwork.publicIdentity}
Block Graffiti       ${blockGraffiti}
Network              ${network}
Memory               ${memoryStatus}
CPU                  ${cpuStatus}
P2P Network          ${peerNetworkStatus}
Mining               ${miningDirectorStatus}
Mem Pool             ${memPoolStatus}
Syncer               ${blockSyncerStatus}
Blockchain           ${blockchainStatus}
Accounts             ${accountStatus}
Telemetry            ${telemetryStatus}
Workers              ${workersStatus}
`;
}
//# sourceMappingURL=status.js.map