"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssetsCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const utils_1 = require("../../utils");
const table_1 = require("../../utils/table");
const MAX_ASSET_METADATA_COLUMN_WIDTH = rust_nodejs_1.ASSET_METADATA_LENGTH + 1;
const MIN_ASSET_METADATA_COLUMN_WIDTH = rust_nodejs_1.ASSET_METADATA_LENGTH / 2 + 1;
const MAX_ASSET_NAME_COLUMN_WIDTH = rust_nodejs_1.ASSET_NAME_LENGTH + 1;
const MIN_ASSET_NAME_COLUMN_WIDTH = rust_nodejs_1.ASSET_NAME_LENGTH / 2 + 1;
const { ...tableFlags } = core_1.CliUx.ux.table.flags();
class AssetsCommand extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(AssetsCommand);
        const account = args.account;
        const client = await this.sdk.connectRpc();
        const response = client.wallet.getAssets({
            account,
        });
        const assetMetadataWidth = flags.extended
            ? MAX_ASSET_METADATA_COLUMN_WIDTH
            : MIN_ASSET_METADATA_COLUMN_WIDTH;
        const assetNameWidth = flags.extended
            ? MAX_ASSET_NAME_COLUMN_WIDTH
            : MIN_ASSET_NAME_COLUMN_WIDTH;
        let showHeader = !flags['no-header'];
        for await (const asset of response.contentStream()) {
            core_1.CliUx.ux.table([asset], {
                name: table_1.TableCols.fixedWidth({
                    header: 'Name',
                    width: assetNameWidth,
                    get: (row) => (0, utils_1.renderAssetNameFromHex)(row.name, {
                        verification: row.verification,
                        outputType: flags.output,
                        verbose: !!flags.verbose,
                        logWarn: this.warn.bind(this),
                    }),
                }),
                id: {
                    header: 'ID',
                    minWidth: rust_nodejs_1.ASSET_ID_LENGTH + 1,
                },
                metadata: table_1.TableCols.fixedWidth({
                    header: 'Metadata',
                    width: assetMetadataWidth,
                    get: (row) => sdk_1.BufferUtils.toHuman(Buffer.from(row.metadata, 'hex')),
                }),
                createdTransactionHash: {
                    header: 'Created Transaction Hash',
                },
                supply: {
                    header: 'Supply',
                    minWidth: 16,
                    get: (row) => row.supply ?? 'NULL',
                },
                creator: {
                    header: 'Creator',
                    minWidth: rust_nodejs_1.PUBLIC_ADDRESS_LENGTH + 1,
                    get: (row) => row.id === rust_nodejs_1.Asset.nativeId().toString('hex')
                        ? sdk_1.BufferUtils.toHuman(Buffer.from(row.creator, 'hex'))
                        : row.creator,
                },
                owner: {
                    header: 'Owner',
                    minWidth: rust_nodejs_1.PUBLIC_ADDRESS_LENGTH + 1,
                    get: (row) => row.id === rust_nodejs_1.Asset.nativeId().toString('hex')
                        ? sdk_1.BufferUtils.toHuman(Buffer.from(row.owner, 'hex'))
                        : row.owner,
                },
            }, {
                printLine: this.log.bind(this),
                ...flags,
                'no-header': !showHeader,
            });
            showHeader = false;
        }
    }
}
AssetsCommand.description = `Display the wallet's assets`;
AssetsCommand.flags = {
    ...flags_1.RemoteFlags,
    ...tableFlags,
};
AssetsCommand.args = [
    {
        name: 'account',
        required: false,
        description: 'Name of the account',
    },
];
exports.AssetsCommand = AssetsCommand;
//# sourceMappingURL=assets.js.map