"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BalanceCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const utils_1 = require("../../utils");
class BalanceCommand extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(BalanceCommand);
        const account = args.account;
        const client = await this.sdk.connectRpc();
        const response = await client.wallet.getAccountBalance({
            account,
            assetId: flags.assetId,
            confirmations: flags.confirmations,
        });
        const asset = (await client.wallet.getAsset({
            account,
            id: response.content.assetId,
            confirmations: flags.confirmations,
        })).content;
        const assetId = response.content.assetId;
        const assetName = (0, utils_1.renderAssetName)((0, sdk_1.isNativeIdentifier)(assetId) ? '$IRON' : assetId, {
            verification: asset.verification,
            verbose: !!flags.verbose,
            logWarn: this.warn.bind(this),
        });
        if (flags.explain) {
            this.explainBalance(response.content, assetName);
            return;
        }
        if (flags.all) {
            this.log(`Account: ${response.content.account}`);
            this.log(`Head Hash: ${response.content.blockHash || 'NULL'}`);
            this.log(`Head Sequence: ${response.content.sequence || 'NULL'}`);
            this.log(`Available:   ${sdk_1.CurrencyUtils.renderIron(response.content.available, true, assetName)}`);
            this.log(`Confirmed:   ${sdk_1.CurrencyUtils.renderIron(response.content.confirmed, true, assetName)}`);
            this.log(`Unconfirmed: ${sdk_1.CurrencyUtils.renderIron(response.content.unconfirmed, true, assetName)}`);
            this.log(`Pending:     ${sdk_1.CurrencyUtils.renderIron(response.content.pending, true, assetName)}`);
            return;
        }
        this.log(`Account: ${response.content.account}`);
        this.log(`Available Balance: ${sdk_1.CurrencyUtils.renderIron(response.content.available, true, assetName)}`);
    }
    explainBalance(response, assetId) {
        const unconfirmed = sdk_1.CurrencyUtils.decode(response.unconfirmed);
        const confirmed = sdk_1.CurrencyUtils.decode(response.confirmed);
        const pending = sdk_1.CurrencyUtils.decode(response.pending);
        const available = sdk_1.CurrencyUtils.decode(response.available);
        const unconfirmedDelta = unconfirmed - confirmed;
        const pendingDelta = pending - unconfirmed;
        this.log(`Account: ${response.account}`);
        this.log(`Your balance is calculated from transactions on the chain through block ${response.blockHash ?? 'NULL'} at sequence ${response.sequence ?? 'NULL'}`);
        this.log('');
        this.log(`Your available balance is made of notes on the chain that are safe to spend`);
        this.log(`Available: ${sdk_1.CurrencyUtils.renderIron(available, true, assetId)}`);
        this.log('');
        this.log('Your confirmed balance includes all notes from transactions on the chain');
        this.log(`Confirmed: ${sdk_1.CurrencyUtils.renderIron(confirmed, true, assetId)}`);
        this.log('');
        this.log(`${response.unconfirmedCount} transactions worth ${sdk_1.CurrencyUtils.renderIron(unconfirmedDelta)} are on the chain within ${response.confirmations} blocks of the head`);
        this.log(`Unconfirmed: ${sdk_1.CurrencyUtils.renderIron(unconfirmed, true, assetId)}`);
        this.log('');
        this.log(`${response.pendingCount} transactions worth ${sdk_1.CurrencyUtils.renderIron(pendingDelta)} are pending and have not been added to the chain`);
        this.log(`Pending: ${sdk_1.CurrencyUtils.renderIron(pending, true, assetId)}`);
    }
}
BalanceCommand.description = 'Display the account balance\n\
  What is the difference between available to spend balance, and balance?\n\
  Available to spend balance is your coins from transactions that have been mined on blocks on your main chain.\n\
  Balance is your coins from all of your transactions, even if they are on forks or not yet included as part of a mined block.';
BalanceCommand.flags = {
    ...flags_1.RemoteFlags,
    explain: core_1.Flags.boolean({
        default: false,
        description: 'Explain your balance',
    }),
    all: core_1.Flags.boolean({
        default: false,
        description: 'Also show unconfirmed balance',
    }),
    confirmations: core_1.Flags.integer({
        required: false,
        description: 'Minimum number of blocks confirmations for a transaction',
    }),
    assetId: core_1.Flags.string({
        required: false,
        description: 'Asset identifier to check the balance for',
    }),
};
BalanceCommand.args = [
    {
        name: 'account',
        required: false,
        description: 'Name of the account to get balance for',
    },
];
exports.BalanceCommand = BalanceCommand;
//# sourceMappingURL=balance.js.map