"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Burn = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const asset_1 = require("../../utils/asset");
const currency_1 = require("../../utils/currency");
const fees_1 = require("../../utils/fees");
const transaction_1 = require("../../utils/transaction");
class Burn extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Burn);
        const client = await this.sdk.connectRpc();
        if (!flags.offline) {
            const status = await client.wallet.getNodeStatus();
            if (!status.content.blockchain.synced) {
                this.log(`Your node must be synced with the Iron Fish network to send a transaction. Please try again later`);
                this.exit(1);
            }
        }
        let account = flags.account;
        if (!account) {
            const response = await client.wallet.getDefaultAccount();
            if (!response.content.account) {
                this.error(`No account is currently active.
           Use ironfish wallet:create <name> to first create an account`);
            }
            account = response.content.account.name;
        }
        let assetId = flags.assetId;
        if (assetId == null) {
            const asset = await (0, asset_1.selectAsset)(client, account, {
                action: 'burn',
                showNativeAsset: false,
                showNonCreatorAsset: true,
                showSingleAssetChoice: true,
                confirmations: flags.confirmations,
            });
            assetId = asset?.id;
        }
        if (assetId == null) {
            this.error(`You must have a custom asset in order to burn.`);
        }
        let amount = flags.amount;
        if (!amount) {
            amount = await (0, currency_1.promptCurrency)({
                client: client,
                required: true,
                text: 'Enter the amount of the custom asset to burn',
                minimum: 1n,
                logger: this.logger,
                balance: {
                    account,
                    confirmations: flags.confirmations,
                    assetId,
                },
            });
        }
        const params = {
            account,
            outputs: [],
            burns: [
                {
                    assetId,
                    value: sdk_1.CurrencyUtils.encode(amount),
                },
            ],
            fee: flags.fee ? sdk_1.CurrencyUtils.encode(flags.fee) : null,
            feeRate: flags.feeRate ? sdk_1.CurrencyUtils.encode(flags.feeRate) : null,
            expiration: flags.expiration,
            confirmations: flags.confirmations,
        };
        let raw;
        if (params.fee === null && params.feeRate === null) {
            raw = await (0, fees_1.selectFee)({
                client,
                transaction: params,
                account: account,
                confirmations: flags.confirmations,
                logger: this.logger,
            });
        }
        else {
            const response = await client.wallet.createTransaction(params);
            const bytes = Buffer.from(response.content.transaction, 'hex');
            raw = sdk_1.RawTransactionSerde.deserialize(bytes);
        }
        if (flags.rawTransaction) {
            this.log('Raw Transaction');
            this.log(sdk_1.RawTransactionSerde.serialize(raw).toString('hex'));
            this.log(`Run "ironfish wallet:post" to post the raw transaction. `);
            this.exit(0);
        }
        if (!flags.confirm && !(await this.confirm(assetId, amount, raw.fee, account))) {
            this.error('Transaction aborted.');
        }
        core_1.CliUx.ux.action.start('Sending the transaction');
        const response = await client.wallet.postTransaction({
            transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            account,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const transaction = new sdk_1.Transaction(bytes);
        core_1.CliUx.ux.action.stop();
        if (response.content.accepted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (response.content.broadcasted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' failed to broadcast`);
        }
        const assetResponse = await client.wallet.getAsset({
            account,
            id: assetId,
            confirmations: flags.confirmations,
        });
        const assetName = sdk_1.BufferUtils.toHuman(Buffer.from(assetResponse.content.name, 'hex'));
        this.log(`Burned asset ${assetName} from ${account}`);
        this.log(`Asset Identifier: ${assetId}`);
        this.log(`Amount: ${sdk_1.CurrencyUtils.renderIron(amount)}`);
        this.log(`Hash: ${transaction.hash().toString('hex')}`);
        this.log(`Fee: ${sdk_1.CurrencyUtils.renderIron(transaction.fee(), true)}`);
        this.log(`\nIf the transaction is mined, it will appear here https://explorer.ironfish.network/transaction/${transaction
            .hash()
            .toString('hex')}`);
        if (flags.watch) {
            this.log('');
            await (0, transaction_1.watchTransaction)({
                client,
                logger: this.logger,
                account,
                hash: transaction.hash().toString('hex'),
            });
        }
    }
    async confirm(assetId, amount, fee, account) {
        this.log(`You are about to burn: ${sdk_1.CurrencyUtils.renderIron(amount, true, assetId)} plus a transaction fee of ${sdk_1.CurrencyUtils.renderIron(fee, true)} with the account ${account}`);
        return core_1.CliUx.ux.confirm('Do you confirm (Y/N)?');
    }
}
Burn.description = 'Burn tokens and decrease supply for a given asset';
Burn.examples = [
    '$ ironfish wallet:burn --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000',
    '$ ironfish wallet:burn --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000 --account otheraccount',
    '$ ironfish wallet:burn --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000 --account otheraccount --fee 0.00000001',
];
Burn.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account to burn from',
    }),
    fee: (0, flags_1.IronFlag)({
        char: 'o',
        description: 'The fee amount in IRON',
        minimum: 1n,
        flagName: 'fee',
    }),
    feeRate: (0, flags_1.IronFlag)({
        char: 'r',
        description: 'The fee rate amount in IRON/Kilobyte',
        minimum: 1n,
        flagName: 'fee rate',
    }),
    amount: (0, flags_1.IronFlag)({
        char: 'a',
        description: 'Amount of coins to burn',
        flagName: 'amount',
    }),
    assetId: core_1.Flags.string({
        char: 'i',
        description: 'Identifier for the asset',
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm without asking',
    }),
    confirmations: core_1.Flags.integer({
        char: 'c',
        description: 'Minimum number of block confirmations needed to include a note. Set to 0 to include all blocks.',
        required: false,
    }),
    rawTransaction: core_1.Flags.boolean({
        default: false,
        description: 'Return raw transaction. Use it to create a transaction but not post to the network',
    }),
    expiration: core_1.Flags.integer({
        char: 'e',
        description: 'The block sequence that the transaction can not be mined after. Set to 0 for no expiration.',
    }),
    offline: core_1.Flags.boolean({
        default: false,
        description: 'Allow offline transaction creation',
    }),
    watch: core_1.Flags.boolean({
        default: false,
        description: 'Wait for the transaction to be confirmed',
    }),
};
exports.Burn = Burn;
//# sourceMappingURL=burn.js.map