"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExportCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const fs_1 = __importDefault(require("fs"));
const json_colorizer_1 = __importDefault(require("json-colorizer"));
const path_1 = __importDefault(require("path"));
const command_1 = require("../../command");
const flags_1 = require("../../flags");
class ExportCommand extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(ExportCommand);
        const { color, local } = flags;
        const account = args.account;
        const exportPath = flags.path;
        const viewOnly = flags.viewonly;
        if (flags.language) {
            flags.mnemonic = true;
        }
        const format = flags.mnemonic
            ? sdk_1.AccountFormat.Mnemonic
            : flags.json
                ? sdk_1.AccountFormat.JSON
                : sdk_1.AccountFormat.Bech32;
        const client = await this.sdk.connectRpc(local);
        const response = await client.wallet.exportAccount({
            account,
            viewOnly,
            format,
            language: flags.language,
        });
        let output = response.content.account;
        if (color && flags.json && !exportPath) {
            output = (0, json_colorizer_1.default)(output);
        }
        if (exportPath) {
            let resolved = this.sdk.fileSystem.resolve(exportPath);
            try {
                const stats = await fs_1.default.promises.stat(resolved);
                if (stats.isDirectory()) {
                    resolved = this.sdk.fileSystem.join(resolved, `ironfish-${account}.txt`);
                }
                if (fs_1.default.existsSync(resolved)) {
                    this.log(`There is already an account backup at ${exportPath}`);
                    const confirmed = await core_1.CliUx.ux.confirm(`\nOverwrite the account backup with new file?\nAre you sure? (Y)es / (N)o`);
                    if (!confirmed) {
                        this.exit(1);
                    }
                }
                await fs_1.default.promises.writeFile(resolved, output);
                this.log(`Exported account ${account} to ${resolved}`);
            }
            catch (err) {
                if (sdk_1.ErrorUtils.isNoEntityError(err)) {
                    await fs_1.default.promises.mkdir(path_1.default.dirname(resolved), { recursive: true });
                    await fs_1.default.promises.writeFile(resolved, output);
                }
                else {
                    throw err;
                }
            }
            return;
        }
        this.log(output);
    }
}
ExportCommand.description = `Export an account`;
ExportCommand.flags = {
    ...flags_1.RemoteFlags,
    [flags_1.ColorFlagKey]: flags_1.ColorFlag,
    local: core_1.Flags.boolean({
        default: false,
        description: 'Export an account without an online node',
    }),
    mnemonic: core_1.Flags.boolean({
        default: false,
        description: 'Export an account to a mnemonic 24 word phrase',
    }),
    language: core_1.Flags.enum({
        description: 'Language to use for mnemonic export',
        required: false,
        options: sdk_1.LanguageUtils.LANGUAGE_KEYS,
    }),
    json: core_1.Flags.boolean({
        default: false,
        description: 'Output the account as JSON, rather than the default bech32',
    }),
    path: core_1.Flags.string({
        description: 'The path to export the account to',
        required: false,
    }),
    viewonly: core_1.Flags.boolean({
        default: false,
        description: 'Export an account as a view-only account',
    }),
};
ExportCommand.args = [
    {
        name: 'account',
        required: false,
        description: 'Name of the account to export',
    },
];
exports.ExportCommand = ExportCommand;
//# sourceMappingURL=export.js.map