"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mint = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const asset_1 = require("../../utils/asset");
const currency_1 = require("../../utils/currency");
const fees_1 = require("../../utils/fees");
const transaction_1 = require("../../utils/transaction");
class Mint extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(Mint);
        const client = await this.sdk.connectRpc();
        if (!flags.offline) {
            const status = await client.wallet.getNodeStatus();
            if (!status.content.blockchain.synced) {
                this.log(`Your node must be synced with the Iron Fish network to send a transaction. Please try again later`);
                this.exit(1);
            }
        }
        let account = flags.account;
        if (!account) {
            const response = await client.wallet.getDefaultAccount();
            if (!response.content.account) {
                this.error(`No account is currently active.
           Use ironfish wallet:create <name> to first create an account`);
            }
            account = response.content.account.name;
        }
        const publicKeyResponse = await client.wallet.getAccountPublicKey({ account });
        const accountPublicKey = publicKeyResponse.content.publicKey;
        if (flags.expiration !== undefined && flags.expiration < 0) {
            this.log('Expiration sequence must be non-negative');
            this.exit(1);
        }
        let assetId = flags.assetId;
        let metadata = flags.metadata;
        let name = flags.name;
        // We can assume the prompt can be skipped if at least one of metadata or
        // name is provided
        let isMintingNewAsset = Boolean(name || metadata);
        if (!assetId && !metadata && !name) {
            isMintingNewAsset = await core_1.CliUx.ux.confirm('Do you want to create a new asset (Y/N)?');
        }
        if (isMintingNewAsset) {
            if (!name) {
                name = await core_1.CliUx.ux.prompt('Enter the name for the new asset', {
                    required: true,
                });
            }
            if (!metadata) {
                metadata = await core_1.CliUx.ux.prompt('Enter metadata for the new asset', {
                    default: '',
                    required: false,
                });
            }
        }
        else if (!assetId) {
            const asset = await (0, asset_1.selectAsset)(client, account, {
                action: 'mint',
                showNativeAsset: false,
                showNonCreatorAsset: false,
                showSingleAssetChoice: true,
                confirmations: flags.confirmations,
            });
            if (!asset) {
                this.error(`You must have an existing asset. Try creating a new one.`);
            }
            assetId = asset.id;
        }
        if (assetId) {
            const isAssetOwner = await this.isAssetOwner(client, assetId, accountPublicKey);
            if (!isAssetOwner) {
                this.error(`The account '${account}' does not own this asset.`);
            }
        }
        let amount = flags.amount;
        if (!amount) {
            amount = await (0, currency_1.promptCurrency)({
                client: client,
                required: true,
                text: 'Enter the amount',
                minimum: 1n,
                logger: this.logger,
            });
        }
        if (flags.transferOwnershipTo) {
            if (!(0, sdk_1.isValidPublicAddress)(flags.transferOwnershipTo)) {
                this.error('transferOwnershipTo must be a valid public address');
            }
        }
        const params = {
            account,
            outputs: [],
            mints: [
                {
                    assetId,
                    name,
                    metadata,
                    value: sdk_1.CurrencyUtils.encode(amount),
                    transferOwnershipTo: flags.transferOwnershipTo,
                },
            ],
            fee: flags.fee ? sdk_1.CurrencyUtils.encode(flags.fee) : null,
            feeRate: flags.feeRate ? sdk_1.CurrencyUtils.encode(flags.feeRate) : null,
            expiration: flags.expiration,
            confirmations: flags.confirmations,
        };
        let raw;
        if (params.fee === null && params.feeRate === null) {
            raw = await (0, fees_1.selectFee)({
                client,
                transaction: params,
                account: account,
                confirmations: flags.confirmations,
                logger: this.logger,
            });
        }
        else {
            const response = await client.wallet.createTransaction(params);
            const bytes = Buffer.from(response.content.transaction, 'hex');
            raw = sdk_1.RawTransactionSerde.deserialize(bytes);
        }
        if (flags.rawTransaction) {
            this.log('Raw Transaction');
            this.log(sdk_1.RawTransactionSerde.serialize(raw).toString('hex'));
            this.log(`Run "ironfish wallet:post" to post the raw transaction. `);
            this.exit(0);
        }
        if (!flags.confirm &&
            !(await this.confirm(account, amount, raw.fee, assetId, name, metadata, flags.transferOwnershipTo))) {
            this.error('Transaction aborted.');
        }
        core_1.CliUx.ux.action.start('Sending the transaction');
        const response = await client.wallet.postTransaction({
            transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            account,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const transaction = new sdk_1.Transaction(bytes);
        core_1.CliUx.ux.action.stop();
        const minted = transaction.mints[0];
        if (response.content.accepted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (response.content.broadcasted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' failed to broadcast`);
        }
        this.log(`Minted asset ${sdk_1.BufferUtils.toHuman(minted.asset.name())} from ${account}`);
        this.log(`Asset Identifier: ${minted.asset.id().toString('hex')}`);
        this.log(`Value: ${sdk_1.CurrencyUtils.renderIron(minted.value, true, minted.asset.id().toString('hex'))}`);
        this.log(`Fee: ${sdk_1.CurrencyUtils.renderIron(transaction.fee(), true)}`);
        this.log(`Hash: ${transaction.hash().toString('hex')}`);
        this.log(`\nIf the transaction is mined, it will appear here https://explorer.ironfish.network/transaction/${transaction
            .hash()
            .toString('hex')}`);
        if (flags.watch) {
            this.log('');
            await (0, transaction_1.watchTransaction)({
                client,
                logger: this.logger,
                account,
                hash: transaction.hash().toString('hex'),
            });
        }
    }
    async confirm(account, amount, fee, assetId, name, metadata, transferOwnershipTo) {
        const nameString = name ? `\nName: ${name}` : '';
        const metadataString = metadata ? `\nMetadata: ${metadata}` : '';
        this.log(`You are about to mint an asset with the account ${account}:${nameString}${metadataString}`);
        this.log(`Amount: ${sdk_1.CurrencyUtils.renderIron(amount, !!assetId, assetId)}`);
        this.log(`Fee: ${sdk_1.CurrencyUtils.renderIron(fee, true)}`);
        if (transferOwnershipTo) {
            this.log(`Ownership of this asset will be transferred to ${transferOwnershipTo}. The current account will no longer have any permission to mint or modify this asset. This cannot be undone.`);
        }
        return core_1.CliUx.ux.confirm('Do you confirm (Y/N)?');
    }
    async isAssetOwner(client, assetId, ownerPublicKey) {
        try {
            const assetResponse = await client.chain.getAsset({ id: assetId });
            if (assetResponse.content.owner === ownerPublicKey) {
                return true;
            }
        }
        catch (e) {
            if (sdk_1.ErrorUtils.isNotFoundError(e)) {
                // Asset doesn't exist yet, so this account would be the creator and owner for the initial mint
                return true;
            }
            else {
                throw e;
            }
        }
        return false;
    }
}
Mint.description = 'Mint tokens and increase supply for a given asset';
Mint.examples = [
    '$ ironfish wallet:mint --metadata "see more here" --name mycoin --amount 1000',
    '$ ironfish wallet:mint --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000',
    '$ ironfish wallet:mint --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000 --account otheraccount',
    '$ ironfish wallet:mint --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000 --account otheraccount --fee 0.00000001',
    '$ ironfish wallet:mint --assetId 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4 --amount 1000 --transferOwnershipTo 0000000000000000000000000000000000000000000000000000000000000000',
];
Mint.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account to mint from',
    }),
    fee: (0, flags_1.IronFlag)({
        char: 'o',
        description: 'The fee amount in IRON',
        minimum: 1n,
        flagName: 'fee',
    }),
    feeRate: (0, flags_1.IronFlag)({
        char: 'r',
        description: 'The fee rate amount in IRON/Kilobyte',
        minimum: 1n,
        flagName: 'fee rate',
    }),
    amount: (0, flags_1.IronFlag)({
        char: 'a',
        description: 'Amount of coins to mint in IRON',
        flagName: 'amount',
    }),
    assetId: core_1.Flags.string({
        char: 'i',
        description: 'Identifier for the asset',
        required: false,
    }),
    metadata: core_1.Flags.string({
        char: 'm',
        description: 'Metadata for the asset',
        required: false,
    }),
    name: core_1.Flags.string({
        char: 'n',
        description: 'Name for the asset',
        required: false,
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm without asking',
    }),
    confirmations: core_1.Flags.integer({
        char: 'c',
        description: 'Minimum number of block confirmations needed to include a note. Set to 0 to include all blocks.',
        required: false,
    }),
    rawTransaction: core_1.Flags.boolean({
        default: false,
        description: 'Return the raw transaction. Used to create a transaction but not post to the network',
    }),
    expiration: core_1.Flags.integer({
        char: 'e',
        description: 'The block sequence that the transaction can not be mined after. Set to 0 for no expiration.',
    }),
    offline: core_1.Flags.boolean({
        default: false,
        description: 'Allow offline transaction creation',
    }),
    watch: core_1.Flags.boolean({
        default: false,
        description: 'Wait for the transaction to be confirmed',
    }),
    transferOwnershipTo: core_1.Flags.string({
        description: 'The public address of the account to transfer ownership of this asset to.',
        required: false,
    }),
};
exports.Mint = Mint;
//# sourceMappingURL=mint.js.map