"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CombineNotesCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const inquirer_1 = __importDefault(require("inquirer"));
const command_1 = require("../../../command");
const flags_1 = require("../../../flags");
const fees_1 = require("../../../utils/fees");
const transaction_1 = require("../../../utils/transaction");
class CombineNotesCommand extends command_1.IronfishCommand {
    async getSpendPostTimeInMs(client, account, noteSize, forceBenchmark) {
        let spendPostTime = this.sdk.internal.get('spendPostTime');
        const spendPostTimeAt = this.sdk.internal.get('spendPostTimeAt');
        const shouldbenchmark = forceBenchmark ||
            spendPostTime <= 0 ||
            Date.now() - spendPostTimeAt > 1000 * 60 * 60 * 24 * 30; // 1 month
        if (shouldbenchmark) {
            spendPostTime = await this.benchmarkSpendPostTime(client, account, noteSize);
            this.sdk.internal.set('spendPostTime', spendPostTime);
            this.sdk.internal.set('spendPostTimeAt', Date.now());
            await this.sdk.internal.save();
        }
        return spendPostTime;
    }
    async benchmarkSpendPostTime(client, account, noteSize) {
        const publicKey = (await client.wallet.getAccountPublicKey({
            account: account,
        })).content.publicKey;
        const notes = await this.fetchNotes(client, account, noteSize, 10);
        core_1.CliUx.ux.action.start('Measuring time to combine 1 note');
        const feeRates = await client.wallet.estimateFeeRates();
        /** Transaction 1: selects 1 note */
        const txn1Params = {
            account: account,
            outputs: [
                {
                    publicAddress: publicKey,
                    amount: sdk_1.CurrencyUtils.encode(BigInt(notes[0].value)),
                    memo: '',
                },
            ],
            fee: null,
            feeRate: null,
            notes: [notes[0].noteHash],
        };
        /** Transaction 2: selects two notes */
        const txn2Params = {
            account: account,
            outputs: [
                {
                    publicAddress: publicKey,
                    amount: sdk_1.CurrencyUtils.encode(BigInt(notes[0].value) + BigInt(notes[1].value)),
                    memo: '',
                },
            ],
            fee: null,
            feeRate: null,
            notes: [notes[0].noteHash, notes[1].noteHash],
        };
        const promisesTxn1 = [];
        const promisesTxn2 = [];
        for (let i = 0; i < 3; i++) {
            promisesTxn1.push(this.measureTransactionPostTime(client, txn1Params, feeRates));
            promisesTxn2.push(this.measureTransactionPostTime(client, txn2Params, feeRates));
        }
        const resultTxn1 = await Promise.all(promisesTxn1);
        const resultTxn2 = await Promise.all(promisesTxn2);
        const delta = Math.ceil((resultTxn2.reduce((acc, curr) => acc + curr, 0) -
            resultTxn1.reduce((acc, curr) => acc + curr, 0)) /
            3);
        core_1.CliUx.ux.action.stop(sdk_1.TimeUtils.renderSpan(delta));
        return delta;
    }
    async measureTransactionPostTime(client, params, feeRates) {
        const response = await client.wallet.createTransaction({
            ...params,
            feeRate: feeRates.content.fast,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const raw = sdk_1.RawTransactionSerde.deserialize(bytes);
        const start = sdk_1.BenchUtils.start();
        await client.wallet.postTransaction({
            transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            broadcast: false,
        });
        return sdk_1.BenchUtils.end(start);
    }
    async fetchNotes(client, account, noteSize, notesToCombine) {
        notesToCombine = Math.max(notesToCombine, 10); // adds a buffer in case the user selects a small number of notes and they get filtered out by noteSize
        const getNotesResponse = await client.wallet.getNotes({
            account,
            pageSize: notesToCombine,
            filter: {
                assetId: rust_nodejs_1.Asset.nativeId().toString('hex'),
                spent: false,
            },
        });
        // filtering notes by noteSize and sorting them by value in ascending order
        const notes = getNotesResponse.content.notes
            .filter((note) => {
            if (!note.index) {
                return false;
            }
            return note.index < noteSize;
        })
            .sort((a, b) => {
            if (a.value < b.value) {
                return -1;
            }
            return 1;
        });
        // must have at least three notes so that you can combine 2 and use another for fees
        if (notes.length < 3) {
            this.log(`Your notes are already combined. You currently have ${notes.length} notes.`);
            this.exit(0);
        }
        return notes;
    }
    async selectNotesToCombine(spendPostTimeMs) {
        const spendsPerMinute = Math.max(Math.floor(60000 / spendPostTimeMs), 2); // minimum of 2 notes per minute in case the spentPostTime is very high
        const low = spendsPerMinute;
        const medium = spendsPerMinute * 5;
        const high = spendsPerMinute * 10;
        const choices = [
            {
                name: `~1 minute: ${low} notes`,
                value: low,
            },
            {
                name: `~5 minutes: ${medium} notes`,
                value: medium,
                default: true,
            },
            {
                name: `~10 minutes: ${high} notes`,
                value: high,
            },
            {
                name: 'Enter a custom number of notes',
                value: null,
            },
        ];
        const result = await inquirer_1.default.prompt([
            {
                name: 'selection',
                message: `Select the number of notes you wish to combine (MAX): `,
                type: 'list',
                choices,
            },
        ]);
        if (result.selection) {
            return result.selection;
        }
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const result = await core_1.CliUx.ux.prompt('Enter the number of notes', {
                required: true,
            });
            const notesToCombine = parseInt(result);
            if (isNaN(notesToCombine)) {
                this.logger.error(`The number of notes must be a number`);
                continue;
            }
            if (notesToCombine > high) {
                this.logger.error(`The number of notes cannot be higher than the ${high}`);
                continue;
            }
            if (notesToCombine < 2) {
                this.logger.error(`The number must be larger than 1`);
                continue;
            }
            return notesToCombine;
        }
    }
    async calculateExpiration(client, spendPostTimeInMs, numberOfNotes) {
        const currentBlockSequence = await this.getCurrentBlockSequence(client);
        let timeLastFiveBlocksInMs = 0;
        let currentBlockTime = new Date((await client.chain.getBlock({
            sequence: currentBlockSequence,
        })).content.block.timestamp);
        for (let i = 0; i < 5; i++) {
            const blockTime = new Date((await client.chain.getBlock({
                sequence: currentBlockSequence - i,
            })).content.block.timestamp);
            timeLastFiveBlocksInMs += currentBlockTime.getTime() - blockTime.getTime();
            currentBlockTime = blockTime;
        }
        const averageBlockTimeInMs = timeLastFiveBlocksInMs / 5;
        const targetBlockTimeInMs = (await client.chain.getConsensusParameters()).content.targetBlockTimeInSeconds * 1000;
        const blockTimeForCalculation = Math.min(averageBlockTimeInMs, targetBlockTimeInMs);
        let expiration = Math.ceil(currentBlockSequence + (spendPostTimeInMs * numberOfNotes * 2) / blockTimeForCalculation);
        const config = await client.config.getConfig();
        if (config.content.transactionExpirationDelta) {
            expiration = Math.max(currentBlockSequence + config.content.transactionExpirationDelta, expiration);
        }
        return expiration;
    }
    async getNoteTreeSize(client) {
        const getCurrentBlock = await client.chain.getChainInfo();
        const currentBlockSequence = parseInt(getCurrentBlock.content.currentBlockIdentifier.index);
        const getBlockResponse = await client.chain.getBlock({
            sequence: currentBlockSequence,
        });
        sdk_1.Assert.isNotNull(getBlockResponse.content.block.noteSize);
        const config = await client.config.getConfig();
        // Adding a buffer to avoid a mismatch between confirmations used to load notes and confirmations used when creating witnesses to spend them
        return getBlockResponse.content.block.noteSize - (config.content.confirmations || 2);
    }
    async getCurrentBlockSequence(client) {
        const getCurrentBlock = await client.chain.getChainInfo();
        const currentBlockSequence = parseInt(getCurrentBlock.content.currentBlockIdentifier.index);
        return currentBlockSequence;
    }
    async start() {
        const { flags } = await this.parse(CombineNotesCommand);
        const client = await this.sdk.connectRpc();
        let to = flags.to;
        let from = flags.account;
        if (!from) {
            const response = await client.wallet.getDefaultAccount();
            if (!response.content.account) {
                this.error(`No account is currently active.
          Use ironfish wallet:create <name> to first create an account`);
            }
            from = response.content.account.name;
        }
        if (!to) {
            const response = await client.wallet.getAccountPublicKey({
                account: from,
            });
            to = response.content.publicKey;
        }
        // the confirmation range in the merkle tree for notes that are safe to use
        const noteSize = await this.getNoteTreeSize(client);
        const spendPostTime = await this.getSpendPostTimeInMs(client, from, noteSize, flags.benchmark);
        let numberOfNotes = flags.notes;
        if (numberOfNotes === undefined) {
            numberOfNotes = await this.selectNotesToCombine(spendPostTime);
        }
        let notes = await this.fetchNotes(client, from, noteSize, numberOfNotes);
        // If the user doesn't have enough notes for their selection, we reduce the number of notes so that
        // the largest note can be used for fees.
        if (notes.length < numberOfNotes) {
            numberOfNotes = notes.length - 1;
        }
        notes = notes.slice(0, numberOfNotes);
        const amount = notes.reduce((acc, note) => acc + BigInt(note.value), 0n);
        const memo = flags.memo?.trim() ??
            (await core_1.CliUx.ux.prompt('Enter the memo (or leave blank)', { required: false }));
        const expiration = await this.calculateExpiration(client, spendPostTime, numberOfNotes);
        const params = {
            account: from,
            outputs: [
                {
                    publicAddress: to,
                    amount: sdk_1.CurrencyUtils.encode(amount),
                    memo,
                },
            ],
            fee: flags.fee ? sdk_1.CurrencyUtils.encode(flags.fee) : null,
            feeRate: flags.feeRate ? sdk_1.CurrencyUtils.encode(flags.feeRate) : null,
            notes: notes.map((note) => note.noteHash),
            expiration,
        };
        let raw;
        if (params.fee === null && params.feeRate === null) {
            raw = await (0, fees_1.selectFee)({
                client,
                transaction: params,
                account: from,
                logger: this.logger,
            });
        }
        else {
            const response = await client.wallet.createTransaction(params);
            const bytes = Buffer.from(response.content.transaction, 'hex');
            raw = sdk_1.RawTransactionSerde.deserialize(bytes);
        }
        (0, transaction_1.displayTransactionSummary)(raw, rust_nodejs_1.Asset.nativeId().toString('hex'), amount, from, to, memo);
        const estimateInMs = Math.ceil(spendPostTime * raw.spends.length);
        this.log(`Time to send: ${sdk_1.TimeUtils.renderSpan(estimateInMs, {
            hideMilliseconds: true,
        })}`);
        if (!flags.confirm) {
            const confirmed = await core_1.CliUx.ux.confirm('Do you confirm (Y/N)?');
            if (!confirmed) {
                this.error('Transaction aborted.');
            }
        }
        const progressBar = core_1.CliUx.ux.progress({
            format: '{title}: [{bar}] {percentage}% | {estimate}',
        });
        const startTime = Date.now();
        progressBar.start(100, 0, {
            title: 'Sending the transaction',
            estimate: sdk_1.TimeUtils.renderSpan(estimateInMs, { hideMilliseconds: true }),
        });
        const timer = setInterval(() => {
            const durationInMs = Date.now() - startTime;
            const timeRemaining = estimateInMs - durationInMs;
            const progress = Math.round((durationInMs / estimateInMs) * 100);
            progressBar.update(progress, {
                estimate: sdk_1.TimeUtils.renderSpan(timeRemaining, { hideMilliseconds: true }),
            });
        }, 1000);
        const response = await client.wallet.postTransaction({
            transaction: sdk_1.RawTransactionSerde.serialize(raw).toString('hex'),
            account: from,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const transaction = new sdk_1.Transaction(bytes);
        clearInterval(timer);
        progressBar.update(100);
        progressBar.stop();
        this.log(`Sending took ${sdk_1.TimeUtils.renderSpan(Date.now() - startTime, {
            hideMilliseconds: true,
        })}`);
        if (response.content.accepted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (response.content.broadcasted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' failed to broadcast`);
        }
        this.log(`Sent ${sdk_1.CurrencyUtils.renderIron(amount, true)} to ${to} from ${from}`);
        this.log(`Hash: ${transaction.hash().toString('hex')}`);
        this.log(`Fee: ${sdk_1.CurrencyUtils.renderIron(transaction.fee(), true)}`);
        this.log(`Memo: ${memo}`);
        this.log(`\nIf the transaction is mined, it will appear here https://explorer.ironfish.network/transaction/${transaction
            .hash()
            .toString('hex')}`);
        if (flags.watch) {
            this.log('');
            await (0, transaction_1.watchTransaction)({
                client,
                logger: this.logger,
                account: from,
                hash: transaction.hash().toString('hex'),
            });
        }
    }
}
CombineNotesCommand.description = `Combine notes into a single note`;
CombineNotesCommand.flags = {
    ...flags_1.RemoteFlags,
    fee: (0, flags_1.IronFlag)({
        char: 'o',
        description: 'The fee amount in IRON',
        minimum: 1n,
        flagName: 'fee',
    }),
    feeRate: (0, flags_1.IronFlag)({
        char: 'r',
        description: 'The fee rate amount in IRON/Kilobyte',
        minimum: 1n,
        flagName: 'fee rate',
    }),
    memo: core_1.Flags.string({
        char: 'm',
        description: 'The memo of transaction',
    }),
    notes: core_1.Flags.integer({
        description: 'How many notes to combine',
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm without asking',
    }),
    watch: core_1.Flags.boolean({
        default: false,
        description: 'Wait for the transaction to be confirmed',
    }),
    to: core_1.Flags.string({
        char: 't',
        description: 'The public address of the recipient',
    }),
    account: core_1.Flags.string({
        char: 'f',
        description: 'The account to send money from',
    }),
    benchmark: core_1.Flags.boolean({
        hidden: true,
        default: false,
        description: 'Force run the benchmark to measure the time to combine 1 note',
    }),
};
exports.CombineNotesCommand = CombineNotesCommand;
//# sourceMappingURL=combine.js.map