"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
class PostCommand extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(PostCommand);
        const transaction = args.transaction;
        const serialized = Buffer.from(transaction, 'hex');
        const raw = sdk_1.RawTransactionSerde.deserialize(serialized);
        const client = await this.sdk.connectRpc();
        if (!flags.confirm) {
            const confirm = await this.confirm(client, raw, flags.account);
            if (!confirm) {
                this.exit(0);
            }
        }
        core_1.CliUx.ux.action.start(`Posting the transaction`);
        const response = await client.wallet.postTransaction({
            transaction,
            account: flags.account,
            broadcast: flags.broadcast,
        });
        core_1.CliUx.ux.action.stop();
        const posted = new sdk_1.Transaction(Buffer.from(response.content.transaction, 'hex'));
        if (response.content.accepted === false) {
            this.warn(`Transaction '${posted.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (response.content.broadcasted === false) {
            this.warn(`Transaction '${posted.hash().toString('hex')}' failed to broadcast`);
        }
        this.log(`Posted transaction with hash ${posted.hash().toString('hex')}\n`);
        this.log(response.content.transaction);
        if (!flags.broadcast) {
            this.log(`\nRun "ironfish wallet:transaction:add" to add the transaction to your wallet`);
        }
    }
    async confirm(client, raw, account) {
        if (!account) {
            const response = await client.wallet.getDefaultAccount();
            if (response.content.account) {
                account = response.content.account.name;
            }
        }
        if (!account) {
            this.error('Can not find an account to confirm the transaction');
        }
        let spending = 0n;
        for (const output of raw.outputs) {
            spending += output.note.value();
        }
        this.log(`You are about to post a transaction that sends ${sdk_1.CurrencyUtils.renderIron(spending, true)}, with ${raw.mints.length} mints and ${raw.burns.length} burns with a fee ${sdk_1.CurrencyUtils.renderIron(raw.fee, true)} using account ${account}`);
        return core_1.CliUx.ux.confirm('Do you want to post this (Y/N)?');
    }
}
PostCommand.summary = 'Post a raw transaction';
PostCommand.description = `Use this command to post a raw transaction.
   The output is a finalized posted transaction.`;
PostCommand.examples = [
    '$ ironfish wallet:post 618c098d8d008c9f78f6155947014901a019d9ec17160dc0f0d1bb1c764b29b4...',
];
PostCommand.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        description: 'The account that created the raw transaction',
        required: false,
    }),
    confirm: core_1.Flags.boolean({
        default: false,
        description: 'Confirm without asking',
    }),
    broadcast: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Broadcast the transaction after posting',
    }),
};
PostCommand.args = [
    {
        name: 'transaction',
        required: true,
        description: 'The raw transaction in hex encoding',
    },
];
exports.PostCommand = PostCommand;
//# sourceMappingURL=post.js.map