"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RescanCommand = void 0;
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const utils_1 = require("../../utils");
class RescanCommand extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(RescanCommand);
        const { follow, local, from } = flags;
        if (local && !follow) {
            this.error('You cannot pass both --local and --no-follow');
        }
        const client = await this.sdk.connectRpc(local);
        core_1.CliUx.ux.action.start('Asking node to start scanning', undefined, {
            stdout: true,
        });
        const response = client.wallet.rescanAccountStream({ follow, from });
        const speed = new sdk_1.Meter();
        const progress = core_1.CliUx.ux.progress({
            format: 'Scanning Blocks: [{bar}] {value}/{total} {percentage}% {speed}/sec | {estimate}',
        });
        let started = false;
        let lastSequence = 0;
        try {
            for await (const { endSequence, sequence } of response.contentStream()) {
                if (!started) {
                    core_1.CliUx.ux.action.stop();
                    speed.start();
                    progress.start(endSequence, 0);
                    started = true;
                }
                const completed = sequence - lastSequence;
                lastSequence = sequence;
                speed.add(completed);
                progress.increment(completed);
                progress.update({
                    estimate: sdk_1.TimeUtils.renderEstimate(sequence, endSequence, speed.rate1m),
                    speed: speed.rate1s.toFixed(0),
                });
            }
        }
        catch (error) {
            progress?.stop();
            speed.stop();
            if ((0, utils_1.hasUserResponseError)(error)) {
                this.error(error.codeMessage);
            }
            throw error;
        }
        speed.stop();
        progress?.stop();
        this.log(follow ? 'Scanning Complete' : 'Scan started in background');
    }
}
RescanCommand.description = `Rescan the blockchain for transactions. Clears wallet disk caches before rescanning.`;
RescanCommand.flags = {
    ...flags_1.RemoteFlags,
    follow: core_1.Flags.boolean({
        char: 'f',
        default: true,
        description: 'Follow the rescan live, or attach to an already running rescan',
        allowNo: true,
    }),
    local: core_1.Flags.boolean({
        default: false,
        description: 'Force the rescan to not connect via RPC',
    }),
    from: core_1.Flags.integer({
        description: 'Sequence to start account rescan from',
        hidden: true,
    }),
};
exports.RescanCommand = RescanCommand;
//# sourceMappingURL=rescan.js.map