"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../../command");
const flags_1 = require("../../../flags");
class TransactionCommand extends command_1.IronfishCommand {
    async start() {
        const { args } = await this.parse(TransactionCommand);
        const hash = args.hash;
        const account = args.account;
        const client = await this.sdk.connectRpc();
        const response = await client.wallet.getAccountTransaction({
            account,
            hash,
        });
        if (!response.content.transaction) {
            this.log(`No transaction found by hash ${hash}`);
            return;
        }
        // by default the notes and spends should be returned
        sdk_1.Assert.isNotUndefined(response.content.transaction.notes);
        sdk_1.Assert.isNotUndefined(response.content.transaction.spends);
        this.log(`Transaction: ${hash}`);
        this.log(`Account: ${response.content.account}`);
        this.log(`Status: ${response.content.transaction.status}`);
        this.log(`Type: ${response.content.transaction.type}`);
        this.log(`Timestamp: ${sdk_1.TimeUtils.renderString(response.content.transaction.timestamp)}`);
        this.log(`Fee: ${sdk_1.CurrencyUtils.renderIron(response.content.transaction.fee, true)}`);
        if (response.content.transaction.blockHash && response.content.transaction.blockSequence) {
            this.log(`Block Hash: ${response.content.transaction.blockHash}`);
            this.log(`Block Sequence: ${response.content.transaction.blockSequence}`);
        }
        this.log(`Notes Count: ${response.content.transaction.notes.length}`);
        this.log(`Spends Count: ${response.content.transaction.spends.length}`);
        this.log(`Mints Count: ${response.content.transaction.mints.length}`);
        this.log(`Burns Count: ${response.content.transaction.burns.length}`);
        this.log(`Sender: ${response.content.transaction.notes[0].sender}`);
        if (response.content.transaction.notes.length > 0) {
            this.log(`\n---Notes---\n`);
            const noteAssetPairs = [];
            for (const note of response.content.transaction.notes) {
                const asset = await client.wallet.getAsset({
                    id: note.assetId,
                });
                noteAssetPairs.push({
                    note,
                    asset: asset.content,
                });
            }
            core_1.CliUx.ux.table(noteAssetPairs, {
                amount: {
                    header: 'Amount',
                    get: ({ note }) => sdk_1.CurrencyUtils.renderIron(note.value),
                },
                assetName: {
                    header: 'Asset Name',
                    get: ({ asset }) => sdk_1.BufferUtils.toHuman(Buffer.from(asset.name, 'hex')),
                },
                assetId: {
                    header: 'Asset Id',
                },
                isSpent: {
                    header: 'Spent',
                    get: ({ note }) => (!note.owner ? '?' : note.spent ? `✔` : `x`),
                },
                memo: {
                    header: 'Memo',
                },
                owner: {
                    header: 'Owner Address',
                    get: ({ note }) => note.owner,
                },
            });
        }
        if (response.content.transaction.spends.length > 0) {
            this.log(`\n---Spends---\n`);
            core_1.CliUx.ux.table(response.content.transaction.spends, {
                size: {
                    header: 'Size',
                    get: (spend) => spend.size,
                },
                nullifier: {
                    header: 'Nullifier',
                    get: (spend) => spend.nullifier,
                },
                commitmment: {
                    header: 'Commitment',
                    get: (spend) => spend.commitment,
                },
            });
        }
        if (response.content.transaction.assetBalanceDeltas) {
            this.log(`\n---Asset Balance Deltas---\n`);
            core_1.CliUx.ux.table(response.content.transaction.assetBalanceDeltas, {
                assetId: {
                    header: 'Asset ID',
                },
                delta: {
                    header: 'Balance Change',
                },
            });
        }
    }
}
TransactionCommand.description = `Display an account transaction`;
TransactionCommand.flags = {
    ...flags_1.RemoteFlags,
};
TransactionCommand.args = [
    {
        name: 'hash',
        parse: (input) => Promise.resolve(input.trim()),
        required: true,
        description: 'Hash of the transaction',
    },
    {
        name: 'account',
        required: false,
        description: 'Name of the account',
    },
];
exports.TransactionCommand = TransactionCommand;
//# sourceMappingURL=index.js.map