"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionsCommand = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../command");
const flags_1 = require("../../flags");
const utils_1 = require("../../utils");
const table_1 = require("../../utils/table");
const { sort: _, ...tableFlags } = core_1.CliUx.ux.table.flags();
class TransactionsCommand extends command_1.IronfishCommand {
    async start() {
        const { flags, args } = await this.parse(TransactionsCommand);
        const account = args.account;
        const format = flags.csv || flags.output === 'csv'
            ? table_1.Format.csv
            : flags.output === 'json'
                ? table_1.Format.json
                : flags.output === 'yaml'
                    ? table_1.Format.yaml
                    : table_1.Format.cli;
        const client = await this.sdk.connectRpc();
        const response = client.wallet.getAccountTransactionsStream({
            account,
            hash: flags.hash,
            sequence: flags.sequence,
            limit: flags.limit,
            offset: flags.offset,
            confirmations: flags.confirmations,
            notes: flags.notes,
        });
        const columns = this.getColumns(flags.extended, flags.notes, format);
        let showHeader = !flags['no-header'];
        let hasTransactions = false;
        for await (const transaction of response.contentStream()) {
            let transactionRows;
            if (flags.notes) {
                sdk_1.Assert.isNotUndefined(transaction.notes);
                const assetLookup = await (0, utils_1.getAssetsByIDs)(client, transaction.notes.map((n) => n.assetId) || [], account, flags.confirmations);
                transactionRows = this.getTransactionRowsByNote(assetLookup, transaction, format);
            }
            else {
                const assetLookup = await (0, utils_1.getAssetsByIDs)(client, transaction.assetBalanceDeltas.map((d) => d.assetId), account, flags.confirmations);
                transactionRows = this.getTransactionRows(assetLookup, transaction, format);
            }
            core_1.CliUx.ux.table(transactionRows, columns, {
                printLine: this.log.bind(this),
                ...flags,
                'no-header': !showHeader,
            });
            showHeader = false;
            hasTransactions = true;
        }
        if (!hasTransactions) {
            this.log('No transactions found');
        }
    }
    getTransactionRows(assetLookup, transaction, format) {
        const nativeAssetId = rust_nodejs_1.Asset.nativeId().toString('hex');
        const assetBalanceDeltas = transaction.assetBalanceDeltas.sort((d) => d.assetId === nativeAssetId ? -1 : 1);
        const feePaid = transaction.type === sdk_1.TransactionType.SEND ? BigInt(transaction.fee) : 0n;
        const transactionRows = [];
        let assetCount = assetBalanceDeltas.length;
        for (const [index, { assetId, delta }] of assetBalanceDeltas.entries()) {
            const asset = assetLookup[assetId];
            let amount = BigInt(delta);
            if (assetId === rust_nodejs_1.Asset.nativeId().toString('hex')) {
                if (transaction.type === sdk_1.TransactionType.SEND) {
                    amount += feePaid;
                }
                // exclude the native asset in cli output if no amount was sent/received
                // and it was not the only asset exchanged
                if (format === table_1.Format.cli && amount === 0n && assetCount > 1) {
                    assetCount -= 1;
                    continue;
                }
            }
            const group = this.getRowGroup(index, assetCount, transactionRows.length);
            // include full transaction details in first row or non-cli-formatted output
            if (transactionRows.length === 0 || format !== table_1.Format.cli) {
                transactionRows.push({
                    ...transaction,
                    group,
                    assetId,
                    assetName: asset.name,
                    amount,
                    feePaid,
                });
            }
            else {
                transactionRows.push({
                    group,
                    assetId,
                    assetName: asset.name,
                    amount,
                });
            }
        }
        return transactionRows;
    }
    getTransactionRowsByNote(assetLookup, transaction, format) {
        sdk_1.Assert.isNotUndefined(transaction.notes);
        const transactionRows = [];
        const nativeAssetId = rust_nodejs_1.Asset.nativeId().toString('hex');
        const notes = transaction.notes.sort((n) => (n.assetId === nativeAssetId ? -1 : 1));
        const noteCount = transaction.notes.length;
        const feePaid = transaction.type === sdk_1.TransactionType.SEND ? BigInt(transaction.fee) : 0n;
        for (const [index, note] of notes.entries()) {
            const amount = BigInt(note.value);
            const assetId = note.assetId;
            const assetName = assetLookup[note.assetId].name;
            const sender = note.sender;
            const recipient = note.owner;
            const group = this.getRowGroup(index, noteCount, transactionRows.length);
            // include full transaction details in first row or non-cli-formatted output
            if (transactionRows.length === 0 || format !== table_1.Format.cli) {
                transactionRows.push({
                    ...transaction,
                    group,
                    assetId,
                    assetName,
                    amount,
                    feePaid,
                    sender,
                    recipient,
                });
            }
            else {
                transactionRows.push({
                    group,
                    assetId,
                    assetName,
                    amount,
                    sender,
                    recipient,
                });
            }
        }
        return transactionRows;
    }
    getColumns(extended, notes, format) {
        let columns = {
            timestamp: table_1.TableCols.timestamp({
                streaming: true,
            }),
            status: {
                header: 'Status',
                minWidth: 12,
            },
            type: {
                header: 'Type',
                minWidth: 8,
            },
            hash: {
                header: 'Hash',
                minWidth: 32,
            },
            notesCount: {
                header: 'Notes',
                minWidth: 5,
                extended: true,
            },
            spendsCount: {
                header: 'Spends',
                minWidth: 5,
                extended: true,
            },
            mintsCount: {
                header: 'Mints',
                minWidth: 5,
                extended: true,
            },
            burnsCount: {
                header: 'Burns',
                minWidth: 5,
                extended: true,
            },
            expiration: {
                header: 'Expiration',
            },
            submittedSequence: {
                header: 'Submitted Sequence',
            },
            feePaid: {
                header: 'Fee Paid ($IRON)',
                get: (row) => row.feePaid && row.feePaid !== 0n ? sdk_1.CurrencyUtils.renderIron(row.feePaid) : '',
            },
            ...table_1.TableCols.asset({ extended, format }),
            amount: {
                header: 'Amount',
                get: (row) => {
                    sdk_1.Assert.isNotUndefined(row.amount);
                    return sdk_1.CurrencyUtils.renderIron(row.amount);
                },
                minWidth: 16,
            },
        };
        if (notes) {
            columns = {
                ...columns,
                sender: {
                    header: 'Sender Address',
                },
                recipient: {
                    header: 'Recipient Address',
                },
            };
        }
        if (format === table_1.Format.cli) {
            columns = {
                group: {
                    header: '',
                    minWidth: 3,
                },
                ...columns,
            };
        }
        return columns;
    }
    getRowGroup(index, assetCount, assetRowCount) {
        if (assetCount > 1) {
            if (assetRowCount === 0) {
                return '┏';
            }
            else if (assetRowCount > 0 && index < assetCount - 1) {
                return '┣';
            }
            else if (assetRowCount > 0 && index === assetCount - 1) {
                return '┗';
            }
        }
        return '';
    }
}
TransactionsCommand.description = `Display the account transactions`;
TransactionsCommand.flags = {
    ...flags_1.RemoteFlags,
    ...tableFlags,
    hash: core_1.Flags.string({
        char: 't',
        description: 'Transaction hash to get details for',
    }),
    sequence: core_1.Flags.integer({
        char: 's',
        description: 'Block sequence to get transactions for',
    }),
    limit: core_1.Flags.integer({
        description: 'Number of latest transactions to get details for',
    }),
    offset: core_1.Flags.integer({
        description: 'Number of latest transactions to skip',
    }),
    confirmations: core_1.Flags.integer({
        description: 'Number of block confirmations needed to confirm a transaction',
    }),
    notes: core_1.Flags.boolean({
        default: false,
        description: 'Include data from transaction output notes',
    }),
};
TransactionsCommand.args = [
    {
        name: 'account',
        required: false,
        description: 'Name of the account',
    },
];
exports.TransactionsCommand = TransactionsCommand;
//# sourceMappingURL=transactions.js.map