"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DownloadedSnapshot = exports.SnapshotDownloader = exports.getDefaultManifestUrl = exports.DEFAULT_TESTNET_MANIFEST_URL = exports.DEFAULT_MAINNET_MANIFEST_URL = void 0;
const sdk_1 = require("@ironfish/sdk");
const axios_1 = __importDefault(require("axios"));
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const tar_1 = __importDefault(require("tar"));
exports.DEFAULT_MAINNET_MANIFEST_URL = `https://snapshots.ironfish.network/manifest.json`;
exports.DEFAULT_TESTNET_MANIFEST_URL = `https://testnet.snapshots.ironfish.network/manifest.json`;
const getDefaultManifestUrl = (networkId) => {
    switch (networkId) {
        case 0:
            return exports.DEFAULT_TESTNET_MANIFEST_URL;
        case 1:
            return exports.DEFAULT_MAINNET_MANIFEST_URL;
        default:
            return null;
    }
};
exports.getDefaultManifestUrl = getDefaultManifestUrl;
const tryParseUrl = (url) => {
    try {
        return new URL(url);
    }
    catch (_) {
        return null;
    }
};
const getSnapshotUrl = (manifestUrl, manifest) => {
    const snapshotUrl = tryParseUrl(manifest.file_name)?.toString();
    if (snapshotUrl) {
        return snapshotUrl;
    }
    // Snapshot URL is not absolute so use a relative URL from the manifest
    const url = new URL(manifestUrl);
    const parts = url.pathname.split('/').filter((s) => !!s.trim());
    parts.pop();
    parts.push(manifest.file_name);
    url.pathname = parts.join('/');
    return url.toString();
};
async function matchesChecksum(file, checksum) {
    const hasher = crypto_1.default.createHash('sha256');
    await new Promise((resolve, reject) => {
        const stream = fs_1.default.createReadStream(file);
        stream.on('end', resolve);
        stream.on('error', reject);
        stream.pipe(hasher, { end: false });
    });
    const fileCheckSum = hasher.digest().toString('hex');
    return fileCheckSum === checksum;
}
class SnapshotDownloader {
    constructor(manifestUrl, dest, nodeChainDBVersion) {
        this.nodeChainDBVersion = nodeChainDBVersion;
        this.manifestUrl = manifestUrl;
        this.dest = dest;
    }
    async manifest() {
        if (this._manifest) {
            return this._manifest;
        }
        const manifest = (await axios_1.default.get(this.manifestUrl)).data;
        this._manifest = manifest;
        return manifest;
    }
    async snapshotURL() {
        const manifest = await this.manifest();
        return getSnapshotUrl(this.manifestUrl, manifest);
    }
    async snapshotPath() {
        const manifest = await this.manifest();
        return path_1.default.join(this.dest, manifest.file_name);
    }
    async download(onDownloadProgress) {
        const manifest = await this.manifest();
        const snapshotPath = await this.snapshotPath();
        if (manifest.database_version > this.nodeChainDBVersion) {
            throw new Error(`This snapshot is from a later database version (${manifest.database_version}) than your node (${this.nodeChainDBVersion}). Aborting import.`);
        }
        let downloaded = 0;
        const statResult = await promises_1.default.stat(snapshotPath).catch(() => null);
        if (statResult?.isFile()) {
            downloaded = statResult.size;
        }
        if (downloaded >= manifest.file_size) {
            return;
        }
        const idleTimeout = 30000;
        let idleLastChunk = performance.now();
        const idleCancelSource = axios_1.default.CancelToken.source();
        const idleInterval = setInterval(() => {
            const timeSinceLastChunk = performance.now() - idleLastChunk;
            if (timeSinceLastChunk > idleTimeout) {
                clearInterval(idleInterval);
                idleCancelSource.cancel(`Download timed out after ${sdk_1.TimeUtils.renderSpan(timeSinceLastChunk)}`);
            }
        }, idleTimeout);
        const snapshotUrl = await this.snapshotURL();
        const response = await (0, axios_1.default)({
            method: 'GET',
            responseType: 'stream',
            url: snapshotUrl,
            cancelToken: idleCancelSource.token,
            headers: {
                range: `bytes=${downloaded}-`,
            },
        });
        const resumingDownload = response.data.statusCode === 206;
        const writer = fs_1.default.createWriteStream(snapshotPath, {
            flags: resumingDownload ? 'a' : 'w',
        });
        downloaded = resumingDownload ? downloaded : 0;
        await new Promise((resolve, reject) => {
            const onWriterError = (e) => {
                writer.removeListener('close', onWriterClose);
                writer.removeListener('error', onWriterError);
                reject(e);
            };
            const onWriterClose = () => {
                writer.removeListener('close', onWriterClose);
                writer.removeListener('error', onWriterError);
                resolve();
            };
            writer.on('error', onWriterError);
            writer.on('close', onWriterClose);
            response.data.on('error', (e) => {
                writer.destroy(e);
            });
            response.data.on('end', () => {
                writer.close();
            });
            response.data.on('data', (chunk) => {
                writer.write(chunk);
                onDownloadProgress(downloaded, downloaded + chunk.length);
                downloaded += chunk.length;
                idleLastChunk = performance.now();
            });
        })
            .catch((error) => {
            if (idleCancelSource.token.reason?.message) {
                throw new Error(idleCancelSource.token.reason?.message);
            }
            else {
                throw new Error(`Error while downloading snapshot file: ${sdk_1.ErrorUtils.renderError(error)}`);
            }
        })
            .finally(() => {
            clearInterval(idleInterval);
        });
    }
    async verifyChecksum(options) {
        const manifest = await this.manifest();
        const destination = path_1.default.join(this.dest, manifest.file_name);
        const matches = await matchesChecksum(destination, manifest.checksum);
        if (!matches) {
            if (options.cleanup) {
                await promises_1.default.rm(destination, { recursive: true, force: true });
            }
            return null;
        }
        return destination;
    }
}
exports.SnapshotDownloader = SnapshotDownloader;
class DownloadedSnapshot {
    constructor(sdk, file) {
        this.sdk = sdk;
        this.file = file;
    }
    get chainDatabasePath() {
        return this.sdk.fileSystem.resolve(this.sdk.config.chainDatabasePath);
    }
    get snapshotDatabasePath() {
        return this.sdk.fileSystem.join(this.sdk.config.tempDir, 'snapshot');
    }
    async unzip(onEntry) {
        await promises_1.default.mkdir(this.snapshotDatabasePath, { recursive: true });
        let totalEntries = 0;
        let extracted = 0;
        tar_1.default.list({
            file: this.file,
            onentry: (_) => onEntry(++totalEntries, extracted, extracted),
        });
        await tar_1.default.extract({
            file: this.file,
            C: this.snapshotDatabasePath,
            strip: 1,
            strict: true,
            onentry: (_) => onEntry(totalEntries, extracted, ++extracted),
        });
    }
    async replaceDatabase() {
        await promises_1.default.rm(this.chainDatabasePath, { recursive: true, force: true, maxRetries: 10 });
        await promises_1.default.rename(this.snapshotDatabasePath, this.chainDatabasePath);
    }
}
exports.DownloadedSnapshot = DownloadedSnapshot;
//# sourceMappingURL=snapshot.js.map