"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CeremonyClient = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const net_1 = __importDefault(require("net"));
class CeremonyClient {
    constructor(options) {
        this.stopPromise = null;
        this.stopResolve = null;
        this.onJoined = new sdk_1.Event();
        this.onInitiateUpload = new sdk_1.Event();
        this.onInitiateContribution = new sdk_1.Event();
        this.onContributionVerified = new sdk_1.Event();
        this.onStopRetry = new sdk_1.Event();
        this.onDisconnect = () => {
            this.stop(false);
            this.socket.off('error', this.onError);
            this.socket.off('close', this.onDisconnect);
        };
        this.onError = (error) => {
            this.logger.error(`Server error ${sdk_1.ErrorUtils.renderError(error)}`);
        };
        this.host = options.host;
        this.port = options.port;
        this.logger = options.logger;
        this.messageBuffer = new sdk_1.MessageBuffer('\n');
        this.socket = new net_1.default.Socket();
        this.socket.on('data', (data) => void this.onData(data));
    }
    async start() {
        this.stopPromise = new Promise((r) => (this.stopResolve = r));
        const error = await connectSocket(this.socket, this.host, this.port)
            .then(() => null)
            .catch((e) => sdk_1.ErrorUtils.renderError(e));
        if (error === null) {
            this.socket.on('error', this.onError);
            this.socket.on('close', this.onDisconnect);
        }
        return error;
    }
    stop(stopRetries) {
        this.socket.end();
        this.stopResolve && this.stopResolve({ stopRetries });
        this.stopPromise = null;
        this.stopResolve = null;
    }
    waitForStop() {
        sdk_1.Assert.isNotNull(this.stopPromise, 'Cannot wait for stop before starting');
        return this.stopPromise;
    }
    contributionComplete() {
        this.send({ method: 'contribution-complete' });
    }
    join(name, token) {
        this.send({ method: 'join', name, token });
    }
    uploadComplete() {
        this.send({ method: 'upload-complete' });
    }
    send(message) {
        this.socket.write(JSON.stringify(message) + '\n');
    }
    onData(data) {
        this.messageBuffer.write(data);
        for (const message of this.messageBuffer.readMessages()) {
            let parsedMessage;
            try {
                parsedMessage = JSON.parse(message);
            }
            catch {
                this.logger.debug(`Received unknown message: ${message}`);
                return;
            }
            if (parsedMessage.method === 'joined') {
                this.onJoined.emit({
                    queueLocation: parsedMessage.queueLocation,
                    estimate: parsedMessage.estimate,
                });
            }
            else if (parsedMessage.method === 'initiate-upload') {
                this.onInitiateUpload.emit({ uploadLink: parsedMessage.uploadLink });
            }
            else if (parsedMessage.method === 'initiate-contribution') {
                this.onInitiateContribution.emit(parsedMessage);
            }
            else if (parsedMessage.method === 'contribution-verified') {
                this.onContributionVerified.emit(parsedMessage);
            }
            else if (parsedMessage.method === 'disconnect') {
                this.onStopRetry.emit({ error: parsedMessage.error });
            }
            else {
                this.logger.info(`Received message: ${message}`);
            }
        }
    }
}
exports.CeremonyClient = CeremonyClient;
// Transform net.Socket.connect() callback into a nicer promise style interface
function connectSocket(socket, host, port) {
    return new Promise((resolve, reject) => {
        const onConnect = () => {
            socket.off('connect', onConnect);
            socket.off('error', onError);
            resolve();
        };
        const onError = (error) => {
            socket.off('connect', onConnect);
            socket.off('error', onError);
            reject(error);
        };
        socket.on('error', onError);
        socket.on('connect', onConnect);
        socket.connect(port, host);
    });
}
//# sourceMappingURL=client.js.map