"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CeremonyServer = void 0;
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const promises_1 = __importDefault(require("fs/promises"));
const net_1 = __importDefault(require("net"));
const path_1 = __importDefault(require("path"));
const uuid_1 = require("uuid");
const utils_1 = require("../utils");
const schema_1 = require("./schema");
class CeremonyServerClient {
    constructor(options) {
        this.messageBuffer = new sdk_1.MessageBuffer('\n');
        this.id = options.id;
        this.socket = options.socket;
        this.connected = true;
        this.logger = options.logger
            .withTag(`client:${this.id.slice(0, 4)}..${this.id.slice(-4)}`)
            .withTag(`ip:${this.socket.remoteAddress || 'unknown'}`);
    }
    send(message) {
        this.socket.write(JSON.stringify(message) + '\n');
    }
    join(name) {
        this._joined = { name };
    }
    get joined() {
        return this._joined !== undefined;
    }
    get name() {
        return this._joined?.name;
    }
    close(error) {
        if (!this.connected) {
            return;
        }
        this.connected = false;
        this.socket.destroy(error);
    }
}
class CeremonyServer {
    constructor(options) {
        this.stopPromise = null;
        this.stopResolve = null;
        this.queue = [];
        this.privateQueue = [];
        this.currentContributor = null;
        this.logger = options.logger;
        this.host = options.host;
        this.port = options.port;
        this.tempDir = options.tempDir;
        this.s3Bucket = options.s3Bucket;
        this.downloadPrefix = options.downloadPrefix;
        this.s3Client = options.s3Client;
        this.contributionTimeoutMs = options.contributionTimeoutMs;
        this.uploadTimeoutMs = options.uploadTimeoutMs;
        this.presignedExpirationSec = options.presignedExpirationSec;
        this.startDate = options.startDate;
        this.token = options.token;
        this.enableIPBanning = options.enableIPBanning;
        this.server = net_1.default.createServer((s) => this.onConnection(s));
    }
    async getLatestParamName() {
        const paramFileNames = await utils_1.S3Utils.getBucketObjects(this.s3Client, this.s3Bucket);
        const validParams = paramFileNames
            .slice(0)
            .filter((fileName) => /^params_\d{5}$/.test(fileName));
        validParams.sort();
        return validParams[validParams.length - 1];
    }
    totalQueueLength() {
        return this.queue.length + this.privateQueue.length;
    }
    closeClient(client, error, disconnect = false) {
        if (this.currentContributor?.client?.id === client.id) {
            if (this.currentContributor.state === 'VERIFYING') {
                this.currentContributor.client = null;
            }
            else {
                clearTimeout(this.currentContributor.actionTimeout);
                this.currentContributor = null;
                void this.startNextContributor();
            }
        }
        disconnect && client.send({ method: 'disconnect', error: sdk_1.ErrorUtils.renderError(error) });
        client.close(error);
    }
    /** initiate a contributor if one does not already exist */
    async startNextContributor() {
        if (this.currentContributor !== null) {
            return;
        }
        const nextClient = this.privateQueue.shift() || this.queue.shift();
        if (!nextClient) {
            return;
        }
        const contributionTimeout = setTimeout(() => {
            this.closeClient(nextClient, new Error('Failed to complete contribution in time'));
        }, this.contributionTimeoutMs);
        this.currentContributor = {
            state: 'STARTED',
            client: nextClient,
            actionTimeout: contributionTimeout,
        };
        const latestParamName = await this.getLatestParamName();
        const nextParamNumber = parseInt(latestParamName.split('_')[1]) + 1;
        nextClient.logger.info(`Starting contribution ${nextParamNumber}`);
        nextClient.send({
            method: 'initiate-contribution',
            downloadLink: `${this.downloadPrefix}/${latestParamName}`,
            contributionNumber: nextParamNumber,
        });
    }
    async start() {
        // Pre-make the directories to check for access
        await promises_1.default.mkdir(this.tempDir, { recursive: true });
        this.stopPromise = new Promise((r) => (this.stopResolve = r));
        this.server.listen(this.port, this.host);
        this.logger.info(`Server started at ${this.host}:${this.port}`);
    }
    stop() {
        this.server.close();
        this.stopResolve && this.stopResolve();
        this.stopPromise = null;
        this.stopResolve = null;
        this.logger.info(`Server stopped on ${this.host}:${this.port}`);
    }
    async waitForStop() {
        await this.stopPromise;
    }
    onConnection(socket) {
        const client = new CeremonyServerClient({ socket, id: (0, uuid_1.v4)(), logger: this.logger });
        socket.on('data', (data) => void this.onData(client, data));
        socket.on('close', () => this.onDisconnect(client));
        socket.on('error', (e) => this.onDisconnect(client, e));
        const ip = socket.remoteAddress;
        if (this.enableIPBanning &&
            (ip === undefined ||
                this.queue.find((c) => c.socket.remoteAddress === ip) !== undefined ||
                this.privateQueue.find((c) => c.socket.remoteAddress === ip) !== undefined ||
                this.currentContributor?.client?.socket.remoteAddress === ip)) {
            this.closeClient(client, new Error('IP address already in queue'), true);
            return;
        }
    }
    onDisconnect(client, e) {
        this.closeClient(client, e);
        this.queue = this.queue.filter((c) => client.id !== c.id);
        this.privateQueue = this.privateQueue.filter((c) => client.id !== c.id);
        e && client.logger.info(`Disconnected with error: ${sdk_1.ErrorUtils.renderError(e)}`);
        client.logger.info(`(Disconnected) public: ${this.queue.length}, private: ${this.privateQueue.length}`);
    }
    async onData(client, data) {
        client.messageBuffer.write(data);
        for (const message of client.messageBuffer.readMessages()) {
            const result = await sdk_1.YupUtils.tryValidate(schema_1.CeremonyClientMessageSchema, message);
            if (result.error) {
                client.logger.error(`Could not parse client message: ${message}`);
                this.closeClient(client, new Error(`Could not parse message`), true);
                return;
            }
            const parsedMessage = result.result;
            client.logger.info(`Message Received: ${parsedMessage.method}`);
            if (parsedMessage.method === 'join' && !client.joined) {
                if (Date.now() < this.startDate && parsedMessage.token !== this.token) {
                    this.closeClient(client, new Error(`The ceremony does not start until ${new Date(this.startDate).toUTCString()}`), true);
                    return;
                }
                client.join(parsedMessage.name);
                if (parsedMessage.token === this.token) {
                    this.privateQueue.push(client);
                    client.send(this.getJoinedMessage(this.privateQueue.length));
                }
                else {
                    this.queue.push(client);
                    client.send(this.getJoinedMessage(this.totalQueueLength()));
                }
                client.logger.info(`(Connected) public: ${this.queue.length}, private: ${this.privateQueue.length}`);
                void this.startNextContributor();
            }
            else if (parsedMessage.method === 'contribution-complete') {
                await this.handleContributionComplete(client).catch((e) => {
                    client.logger.error(`Error handling contribution-complete: ${sdk_1.ErrorUtils.renderError(e)}`);
                    this.closeClient(client, new Error(`Error generating upload url`));
                });
            }
            else if (parsedMessage.method === 'upload-complete') {
                await this.handleUploadComplete(client).catch((e) => {
                    client.logger.error(`Error handling upload-complete: ${sdk_1.ErrorUtils.renderError(e)}`);
                    this.closeClient(client);
                    if (this.currentContributor?.client == null) {
                        this.currentContributor = null;
                        void this.startNextContributor();
                    }
                });
            }
            else {
                client.logger.error(`Unknown method received: ${message}`);
                this.closeClient(client, new Error(`Unknown method received`));
            }
        }
    }
    async handleContributionComplete(client) {
        if (this.currentContributor?.client?.id !== client.id ||
            this.currentContributor.state !== 'STARTED') {
            throw new Error('contribution-complete message sent but not the current contributor');
        }
        clearTimeout(this.currentContributor.actionTimeout);
        client.logger.info('Generating presigned URL');
        const presignedUrl = await utils_1.S3Utils.getPresignedUploadUrl(this.s3Client, this.s3Bucket, client.id, this.presignedExpirationSec);
        client.logger.info('Sending back presigned URL');
        client.send({
            method: 'initiate-upload',
            uploadLink: presignedUrl,
        });
        this.currentContributor = {
            state: 'UPLOADING',
            actionTimeout: setTimeout(() => {
                this.closeClient(client, new Error('Failed to complete upload in time'));
            }, this.uploadTimeoutMs),
            client: this.currentContributor.client,
        };
    }
    getJoinedMessage(position) {
        const estimate = position * ((this.contributionTimeoutMs + this.uploadTimeoutMs) / 2);
        return { method: 'joined', queueLocation: position, estimate };
    }
    sendUpdatedLocationsToClients() {
        for (const [i, client] of this.privateQueue.entries()) {
            client.send(this.getJoinedMessage(i + 1));
        }
        for (const [i, client] of this.queue.entries()) {
            client.send(this.getJoinedMessage(i + 1 + this.privateQueue.length));
        }
    }
    async handleUploadComplete(client) {
        if (this.currentContributor?.client?.id !== client.id ||
            this.currentContributor.state !== 'UPLOADING') {
            throw new Error('upload-complete message sent but not the current contributor');
        }
        clearTimeout(this.currentContributor.actionTimeout);
        this.currentContributor = {
            state: 'VERIFYING',
            client: this.currentContributor.client,
        };
        client.logger.info('Getting latest contribution from S3');
        const latestParamName = await this.getLatestParamName();
        const nextParamNumber = parseInt(latestParamName.split('_')[1]) + 1;
        const oldParamsDownloadPath = path_1.default.join(this.tempDir, latestParamName);
        const paramsExist = await promises_1.default
            .access(oldParamsDownloadPath)
            .then((_) => true)
            .catch((_) => false);
        const oldParamsPromise = paramsExist
            ? Promise.resolve()
            : utils_1.S3Utils.downloadFromBucket(this.s3Client, this.s3Bucket, latestParamName, oldParamsDownloadPath);
        const newParamsDownloadPath = path_1.default.join(this.tempDir, client.id);
        const newParamsPromise = utils_1.S3Utils.downloadFromBucket(this.s3Client, this.s3Bucket, client.id, newParamsDownloadPath);
        client.logger.info(`Downloading params from S3 to verify`);
        await Promise.all([oldParamsPromise, newParamsPromise]);
        client.logger.info(`Deleting uploaded params from S3`);
        await utils_1.S3Utils.deleteFromBucket(this.s3Client, this.s3Bucket, client.id);
        client.logger.info(`Verifying contribution`);
        const hash = await (0, rust_nodejs_1.verifyTransform)(oldParamsDownloadPath, newParamsDownloadPath);
        client.logger.info(`Uploading verified contribution`);
        const destFile = 'params_' + nextParamNumber.toString().padStart(5, '0');
        const metadata = {
            ...(client.name && { contributorName: encodeURIComponent(client.name) }),
        };
        await utils_1.S3Utils.uploadToBucket(this.s3Client, newParamsDownloadPath, 'application/octet-stream', this.s3Bucket, destFile, client.logger, metadata);
        client.logger.info(`Cleaning up local files`);
        await promises_1.default.rename(newParamsDownloadPath, path_1.default.join(this.tempDir, destFile));
        await promises_1.default.rm(oldParamsDownloadPath);
        client.send({
            method: 'contribution-verified',
            hash,
            downloadLink: `${this.downloadPrefix}/${destFile}`,
            contributionNumber: nextParamNumber,
        });
        client.logger.info(`Contribution ${nextParamNumber} complete`);
        this.currentContributor = null;
        await this.startNextContributor();
        this.sendUpdatedLocationsToClients();
    }
}
exports.CeremonyServer = CeremonyServer;
//# sourceMappingURL=server.js.map