"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAssetsByIDs = exports.selectAsset = exports.compareAssets = exports.renderAssetNameFromHex = exports.renderAssetName = void 0;
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
const chalk_1 = __importDefault(require("chalk"));
const inquirer_1 = __importDefault(require("inquirer"));
function renderAssetName(name, options) {
    if (options?.outputType) {
        // User requested some machine-readable output (like CSV, JSON, or YAML).
        // Do not alter the name in any way.
        return name;
    }
    name = sdk_1.StringUtils.sanitizeString(name);
    switch (options?.verification?.status) {
        case 'verified':
            return chalk_1.default.green(name + '✓');
        case 'unknown':
            if (options?.verbose && options?.logWarn) {
                options.logWarn(`Could not check whether ${name} is a verified asset`);
            }
            return name;
        default:
            return name;
    }
}
exports.renderAssetName = renderAssetName;
function renderAssetNameFromHex(hexName, options) {
    const name = sdk_1.BufferUtils.toHuman(Buffer.from(hexName, 'hex'));
    return renderAssetName(name, options);
}
exports.renderAssetNameFromHex = renderAssetNameFromHex;
function compareAssets(leftName, leftVerification, rightName, rightVerification) {
    const isLeftVerified = leftVerification?.status === 'verified';
    const isRightVerified = rightVerification?.status === 'verified';
    // Sort by verified status first, then by name
    if (isLeftVerified && !isRightVerified) {
        return -1;
    }
    if (!isLeftVerified && isRightVerified) {
        return 1;
    }
    if (leftName < rightName) {
        return -1;
    }
    if (leftName > rightName) {
        return 1;
    }
    return 0;
}
exports.compareAssets = compareAssets;
async function selectAsset(client, account, options) {
    const balancesResponse = await client.wallet.getAccountBalances({
        account: account,
        confirmations: options.confirmations,
    });
    let balances = balancesResponse.content.balances;
    const assetLookup = await getAssetsByIDs(client, balances.map((b) => b.assetId), account, options.confirmations);
    if (!options.showNativeAsset) {
        balances = balances.filter((b) => b.assetId !== rust_nodejs_1.Asset.nativeId().toString('hex'));
    }
    if (!options.showNonCreatorAsset) {
        const accountResponse = await client.wallet.getAccountPublicKey({
            account: account,
        });
        balances = balances.filter((b) => assetLookup[b.assetId].creator === accountResponse.content.publicKey);
    }
    if (balances.length === 0) {
        return undefined;
    }
    if (balances.length === 1 && !options.showSingleAssetChoice) {
        // If there's only one available asset, showing the choices is unnecessary
        return {
            id: balances[0].assetId,
            name: assetLookup[balances[0].assetId].name,
        };
    }
    const choices = balances.map((balance) => {
        const assetName = sdk_1.BufferUtils.toHuman(Buffer.from(assetLookup[balance.assetId].name, 'hex'));
        const name = `${balance.assetId} (${assetName}) (${sdk_1.CurrencyUtils.renderIron(balance.available)})`;
        const value = {
            id: balance.assetId,
            name: assetLookup[balance.assetId].name,
        };
        return { value, name };
    });
    const response = await inquirer_1.default.prompt([
        {
            name: 'asset',
            message: `Select the asset you wish to ${options.action}`,
            type: 'list',
            choices,
        },
    ]);
    return response.asset;
}
exports.selectAsset = selectAsset;
async function getAssetsByIDs(client, assetIds, account, confirmations) {
    assetIds = [...new Set(assetIds)];
    const assets = await Promise.all(assetIds.map((id) => client.wallet.getAsset({ id, account, confirmations })));
    const assetLookup = {};
    assets.forEach((asset) => {
        assetLookup[asset.content.id] = asset.content;
    });
    return assetLookup;
}
exports.getAssetsByIDs = getAssetsByIDs;
//# sourceMappingURL=asset.js.map