"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3Utils = void 0;
const client_cognito_identity_1 = require("@aws-sdk/client-cognito-identity");
const client_s3_1 = require("@aws-sdk/client-s3");
const client_secrets_manager_1 = require("@aws-sdk/client-secrets-manager");
const s3_request_presigner_1 = require("@aws-sdk/s3-request-presigner");
const sdk_1 = require("@ironfish/sdk");
const promises_1 = __importDefault(require("fs/promises"));
const promises_2 = require("stream/promises");
// AWS requires that upload parts be at least 5MB
const MINIMUM_MULTIPART_FILE_SIZE = 5 * 1024 * 1024;
const MAX_MULTIPART_NUM = 10000;
class UploadToBucketError extends Error {
    constructor(message, error) {
        super(message);
        this.name = this.constructor.name;
        this.error = error;
    }
}
class CreateMultipartError extends UploadToBucketError {
}
class UploadMultipartError extends UploadToBucketError {
}
class UploadLastMultipartError extends UploadToBucketError {
}
class UploadReadFileError extends UploadToBucketError {
}
class UploadFailedError extends UploadToBucketError {
}
const R2_SECRET_NAME = 'r2-prod-access-key';
const R2_ENDPOINT = `https://a93bebf26da4c2fe205f71c896afcf89.r2.cloudflarestorage.com`;
async function uploadToBucket(s3, filePath, contentType, bucket, keyName, logger, metadata) {
    const fileHandle = await promises_1.default.open(filePath, 'r');
    const stat = await promises_1.default.stat(filePath);
    const fileSize = stat.size;
    let numParts = MAX_MULTIPART_NUM;
    while (fileSize / numParts < MINIMUM_MULTIPART_FILE_SIZE) {
        numParts /= 2;
    }
    const uploadChunkSize = fileSize / numParts;
    const contentStream = fileHandle.createReadStream();
    const params = {
        Bucket: bucket,
        Key: keyName,
        ContentType: contentType,
        Metadata: metadata,
    };
    const uploadId = await s3
        .send(new client_s3_1.CreateMultipartUploadCommand(params))
        .then((result) => result.UploadId)
        .catch((err) => {
        logger?.debug(`Could not create multipart upload to S3: ${err.message}`);
        throw new CreateMultipartError(`Could not create multipart upload to S3`, err);
    });
    sdk_1.Assert.isNotUndefined(uploadId);
    const uploadPartsPromise = new Promise((resolve, reject) => {
        const partMap = {
            Parts: [],
        };
        let partNum = 1;
        let acc = null;
        contentStream.on('data', (chunk) => {
            if (!acc) {
                acc = chunk;
            }
            else {
                acc = Buffer.concat([acc, chunk]);
            }
            if (acc.length > uploadChunkSize) {
                contentStream.pause();
                const params = {
                    Bucket: bucket,
                    Key: keyName,
                    PartNumber: partNum,
                    UploadId: uploadId,
                    ContentType: contentType,
                    Body: acc,
                };
                s3.send(new client_s3_1.UploadPartCommand(params))
                    .then((result) => {
                    partMap.Parts.push({ ETag: result.ETag, PartNumber: params.PartNumber });
                    partNum += 1;
                    acc = null;
                    contentStream.resume();
                })
                    .catch((err) => {
                    logger?.debug(`Could not upload part to S3 bucket: ${sdk_1.ErrorUtils.renderError(err)}`);
                    reject(new UploadMultipartError('Could not upload part to S3 bucket', err));
                });
            }
        });
        contentStream.on('close', () => {
            if (acc) {
                const params = {
                    Bucket: bucket,
                    Key: keyName,
                    PartNumber: partNum,
                    UploadId: uploadId,
                    ContentType: contentType,
                    Body: acc,
                };
                s3.send(new client_s3_1.UploadPartCommand(params))
                    .then((result) => {
                    partMap.Parts.push({ ETag: result.ETag, PartNumber: params.PartNumber });
                    acc = null;
                    resolve(partMap);
                })
                    .catch((err) => {
                    logger?.debug(`Could not upload last part to S3 bucket: ${sdk_1.ErrorUtils.renderError(err)}`);
                    reject(new UploadLastMultipartError('Could not upload last part to S3 bucket', err));
                });
            }
        });
        contentStream.on('error', (err) => {
            logger?.debug(`Could not read file: ${err.message}; aborting upload to S3...`);
            const params = {
                Bucket: bucket,
                Key: keyName,
                UploadId: uploadId,
            };
            void s3
                .send(new client_s3_1.AbortMultipartUploadCommand(params))
                .then()
                .catch(() => {
                // Ignore error
            });
            reject(new UploadReadFileError(undefined, err));
        });
    });
    const partMap = await uploadPartsPromise;
    logger?.debug(`All parts of snapshot have been uploaded. Finalizing multipart upload. Parts: ${partMap.Parts.length}`);
    const completionParams = {
        Bucket: bucket,
        Key: keyName,
        UploadId: uploadId,
        MultipartUpload: partMap,
    };
    await s3
        .send(new client_s3_1.CompleteMultipartUploadCommand(completionParams))
        .then(() => {
        logger?.debug(`Multipart upload complete.`);
    })
        .catch((err) => {
        throw new UploadFailedError('Could not complete multipart S3 upload', err);
    });
}
async function downloadFromBucket(s3, bucket, keyName, output) {
    const command = new client_s3_1.GetObjectCommand({ Bucket: bucket, Key: keyName });
    const response = await s3.send(command);
    if (response.Body) {
        const fileHandle = await promises_1.default.open(output, 'w');
        const ws = fileHandle.createWriteStream();
        await (0, promises_2.pipeline)(response.Body, ws);
        ws.close();
        await fileHandle.close();
    }
}
async function getPresignedUploadUrl(s3, bucket, keyName, expiresInSeconds) {
    const command = new client_s3_1.PutObjectCommand({
        Bucket: bucket,
        Key: keyName,
    });
    const signedUrl = await (0, s3_request_presigner_1.getSignedUrl)(s3, command, {
        expiresIn: expiresInSeconds,
    });
    return signedUrl;
}
/**
 * Returns an HTTPS URL to a file in S3.
 * https://docs.aws.amazon.com/AmazonS3/latest/userguide/transfer-acceleration-getting-started.html
 * https://docs.aws.amazon.com/AmazonS3/latest/userguide/dual-stack-endpoints.html
 */
function getDownloadUrl(bucket, key, region, options) {
    const dualStack = options?.dualStack ?? false;
    let regionString;
    if (region.accelerated) {
        regionString = dualStack ? 's3-accelerate.dualstack' : 's3-accelerate';
    }
    else {
        regionString = dualStack ? `s3.dualstack.${region.regionCode}` : `s3.${region.regionCode}`;
    }
    return `https://${bucket}.${regionString}.amazonaws.com/${key}`;
}
async function getObjectMetadata(s3, bucket, key) {
    const command = new client_s3_1.HeadObjectCommand({ Bucket: bucket, Key: key });
    const response = await s3.send(command);
    return response;
}
async function getBucketObjects(s3, bucket) {
    let truncated = true;
    let commandParams = { Bucket: bucket };
    const keys = [];
    while (truncated) {
        const command = new client_s3_1.ListObjectsCommand(commandParams);
        const response = await s3.send(command);
        for (const obj of response.Contents || []) {
            if (obj.Key !== undefined) {
                keys.push(obj.Key);
            }
        }
        truncated = response.IsTruncated || false;
        commandParams = { Bucket: bucket, Marker: response.Contents?.slice(-1)[0]?.Key };
    }
    return keys;
}
async function deleteFromBucket(s3Client, bucket, fileName) {
    return s3Client.send(new client_s3_1.DeleteObjectCommand({ Bucket: bucket, Key: fileName }));
}
function getS3Client(useDualstackEndpoint, credentials) {
    const region = 'us-east-1';
    if (credentials) {
        return new client_s3_1.S3Client({
            useAccelerateEndpoint: true,
            useDualstackEndpoint,
            credentials,
            region,
        });
    }
    return new client_s3_1.S3Client({
        useAccelerateEndpoint: true,
        useDualstackEndpoint,
        region,
    });
}
function getR2S3Client(credentials) {
    return new client_s3_1.S3Client({
        region: 'auto',
        endpoint: R2_ENDPOINT,
        credentials: {
            accessKeyId: credentials.r2AccessKeyId,
            secretAccessKey: credentials.r2SecretAccessKey,
        },
    });
}
async function getR2Credentials(region) {
    const client = new client_secrets_manager_1.SecretsManagerClient({ region });
    const command = new client_secrets_manager_1.GetSecretValueCommand({ SecretId: R2_SECRET_NAME });
    const response = await client.send(command);
    if (response.SecretString === undefined) {
        return;
    }
    else {
        return JSON.parse(response.SecretString);
    }
}
async function getCognitoIdentityCredentials() {
    const identityPoolId = 'us-east-1:3ebc542a-6ac4-4c5d-9558-1621eadd2382';
    const cognito = new client_cognito_identity_1.CognitoIdentity({ region: 'us-east-1' });
    const identityResponse = await cognito.getId({ IdentityPoolId: identityPoolId });
    const identityId = identityResponse.IdentityId;
    const credentialsResponse = await cognito.getCredentialsForIdentity({
        IdentityId: identityId,
    });
    const cognitoAccessKeyId = credentialsResponse.Credentials?.AccessKeyId;
    const cognitoSecretAccessKey = credentialsResponse.Credentials?.SecretKey;
    const cognitoSessionToken = credentialsResponse.Credentials?.SessionToken;
    sdk_1.Assert.isNotUndefined(cognitoAccessKeyId);
    sdk_1.Assert.isNotUndefined(cognitoSecretAccessKey);
    sdk_1.Assert.isNotUndefined(cognitoSessionToken);
    return {
        accessKeyId: cognitoAccessKeyId,
        secretAccessKey: cognitoSecretAccessKey,
        sessionToken: cognitoSessionToken,
    };
}
exports.S3Utils = {
    deleteFromBucket,
    downloadFromBucket,
    getBucketObjects,
    getCognitoIdentityCredentials,
    getDownloadUrl,
    getObjectMetadata,
    getPresignedUploadUrl,
    getR2Credentials,
    getR2S3Client,
    getS3Client,
    uploadToBucket,
};
//# sourceMappingURL=s3.js.map