"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableCols = exports.Format = void 0;
const rust_nodejs_1 = require("@ironfish/rust-nodejs");
const sdk_1 = require("@ironfish/sdk");
/**
 * Estimated max length of the longest TimeUtils.renderTime()
 *
 * I chose a date where all time components in en-us have 2 digits
 * this will be wrong in some cases but it's a decent heuristic.
 *
 * 12/25/2024 11:59:59 PM EST
 */
const MAX_TIMESTAMP_LENGTH = sdk_1.TimeUtils.renderString(new Date(2024, 11, 25, 23, 59, 59).getTime()).length;
const MAX_ASSET_NAME_COLUMN_WIDTH = rust_nodejs_1.ASSET_NAME_LENGTH + 1;
const MIN_ASSET_NAME_COLUMN_WIDTH = rust_nodejs_1.ASSET_NAME_LENGTH / 2 + 1;
const timestamp = (options) => {
    const header = options?.header ?? 'Timestamp';
    const field = options?.field ?? 'timestamp';
    // Are you rendering one row at a time by a streaming response?
    const streaming = options?.streaming ?? false;
    // We should only use estimated minWidth if you are streaming
    let minWidth;
    if (streaming && options?.minWidth === undefined) {
        minWidth = MAX_TIMESTAMP_LENGTH;
    }
    else {
        minWidth = options?.minWidth;
    }
    const get = options?.get ??
        ((row) => {
            const value = row[field];
            if (!value) {
                return '';
            }
            return sdk_1.TimeUtils.renderString(Number(value));
        });
    return {
        header,
        get,
        minWidth,
    };
};
const asset = (options) => {
    if (options?.extended || options?.format !== Format.cli) {
        return {
            assetId: {
                header: 'Asset ID',
                get: (row) => row['assetId'],
                minWidth: MAX_ASSET_NAME_COLUMN_WIDTH,
                extended: options?.extended ?? false,
            },
            assetName: {
                header: 'Asset Name',
                get: (row) => {
                    sdk_1.Assert.isString(row.assetName);
                    const assetName = sdk_1.BufferUtils.toHuman(Buffer.from(row.assetName, 'hex'));
                    return truncateCol(assetName, MAX_ASSET_NAME_COLUMN_WIDTH);
                },
                minWidth: MAX_ASSET_NAME_COLUMN_WIDTH,
                extended: options?.extended ?? false,
            },
        };
    }
    else {
        return {
            asset: {
                header: 'Asset',
                get: (row) => {
                    sdk_1.Assert.isString(row.assetName);
                    sdk_1.Assert.isString(row.assetId);
                    const assetName = truncateCol(sdk_1.BufferUtils.toHuman(Buffer.from(row.assetName, 'hex')), MIN_ASSET_NAME_COLUMN_WIDTH);
                    const text = assetName.padEnd(MIN_ASSET_NAME_COLUMN_WIDTH, ' ');
                    return `${text} (${row.assetId.slice(0, 5)})`;
                },
                minWidth: MIN_ASSET_NAME_COLUMN_WIDTH,
                extended: false,
            },
        };
    }
};
const fixedWidth = (options) => {
    return {
        ...options,
        get: (row) => truncateCol(options.get(row), options.width),
        minWidth: options.width,
    };
};
function truncateCol(value, maxWidth) {
    if (maxWidth === null || value.length <= maxWidth) {
        return value;
    }
    return value.slice(0, maxWidth - 1) + '…';
}
var Format;
(function (Format) {
    Format["cli"] = "cli";
    Format["csv"] = "csv";
    Format["json"] = "json";
    Format["yaml"] = "yaml";
})(Format = exports.Format || (exports.Format = {}));
exports.TableCols = { timestamp, asset, fixedWidth };
//# sourceMappingURL=table.js.map