"use strict";
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchTransaction = exports.displayTransactionSummary = void 0;
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
function displayTransactionSummary(transaction, assetId, amount, from, to, memo, logger) {
    logger = logger ?? (0, sdk_1.createRootLogger)();
    const amountString = sdk_1.CurrencyUtils.renderIron(amount, true, assetId);
    const feeString = sdk_1.CurrencyUtils.renderIron(transaction.fee, true);
    const summary = `\
\nTRANSACTION SUMMARY:
From                 ${from}
To                   ${to}
Amount               ${amountString}
Fee                  ${feeString}
Memo                 ${memo}
Outputs              ${transaction.outputs.length}
Spends               ${transaction.spends.length}
Expiration           ${transaction.expiration ? transaction.expiration.toString() : ''}
`;
    logger.log(summary);
}
exports.displayTransactionSummary = displayTransactionSummary;
async function watchTransaction(options) {
    const logger = options.logger ?? (0, sdk_1.createRootLogger)();
    const waitUntil = options.waitUntil ?? sdk_1.TransactionStatus.CONFIRMED;
    const pollFrequencyMs = options.pollFrequencyMs ?? 10000;
    let lastTime = Date.now();
    let last = await options.client.wallet.getAccountTransaction({
        account: options.account,
        hash: options.hash,
        confirmations: options.confirmations,
    });
    const startTime = lastTime;
    let prevStatus = last?.content.transaction?.status ?? 'not found';
    let currentStatus = prevStatus;
    // If the transaction is already in the desired state, return
    if (currentStatus === waitUntil) {
        logger.log(`Transaction ${options.hash} is ${waitUntil}`);
        return;
    }
    logger.log(`Watching transaction ${options.hash}`);
    core_1.CliUx.ux.action.start(`Current Status`);
    const span = sdk_1.TimeUtils.renderSpan(0, { hideMilliseconds: true });
    core_1.CliUx.ux.action.status = `${currentStatus} ${span}`;
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const response = await options.client.wallet.getAccountTransaction({
            account: options.account,
            hash: options.hash,
            confirmations: options.confirmations,
        });
        currentStatus = response?.content.transaction?.status ?? 'not found';
        if (prevStatus !== 'not found' && currentStatus === 'not found') {
            core_1.CliUx.ux.action.stop(`Transaction ${options.hash} deleted while watching it.`);
            break;
        }
        if (currentStatus === prevStatus) {
            const duration = Date.now() - lastTime;
            const span = sdk_1.TimeUtils.renderSpan(duration, { hideMilliseconds: true });
            core_1.CliUx.ux.action.status = `${currentStatus} ${span}`;
            await sdk_1.PromiseUtils.sleep(pollFrequencyMs);
            continue;
        }
        // State has changed
        const now = Date.now();
        const duration = now - lastTime;
        lastTime = now;
        core_1.CliUx.ux.action.stop(`${prevStatus} -> ${currentStatus}: ${sdk_1.TimeUtils.renderSpan(duration, {
            hideMilliseconds: true,
        })}`);
        last = response;
        prevStatus = currentStatus;
        core_1.CliUx.ux.action.start(`Current Status`);
        const span = sdk_1.TimeUtils.renderSpan(0, { hideMilliseconds: true });
        core_1.CliUx.ux.action.status = `${currentStatus} ${span}`;
        if (currentStatus === waitUntil) {
            const duration = now - startTime;
            const span = sdk_1.TimeUtils.renderSpan(duration, { hideMilliseconds: true });
            core_1.CliUx.ux.action.stop(`done after ${span}`);
            break;
        }
    }
}
exports.watchTransaction = watchTransaction;
//# sourceMappingURL=transaction.js.map